// (ships-within-radius x:int y:int radius:double)
static LPtr
FShipsWithinRadius (LPtr p)
{
    int x = argInt(p);
    int y = argInt(p);
    int radius = argInt(p);
    return listFromIdArray(EnumerateShipsWithinRadius(x, y, radius));
}

// (ships-at x:int y:int)
static LPtr
FShipsAt (LPtr p)
{
    int x = argInt(p);
    int y = argInt(p);
    return listFromIdArray(EnumerateShipsAt(x, y));
}

// (mines-within-radius x:int y:int radius:int)
static LPtr
FMinesWithinRadius (LPtr p)
{
    int x = argInt(p);
    int y = argInt(p);
    int radius = argInt(p);
    return listFromIdArray(EnumerateMinesWithinRadius(x, y, radius));
}

// (planet-at-ship x:sid)
static LPtr
FPlanetAtShip (LPtr p)
{
    int x = argSid(p);
    return intOrNull(FindPlanetAtShip(x));
}

// (gravity-planet x:int y:int)
static LPtr
FGravityPlanet (LPtr p)
{
    int x = argInt(p);
    int y = argInt(p);
    return intOrNull(FindGravityPlanet(x, y));
}

// (planets-within-radius x:int y:int radius:int)
static LPtr
FPlanetsWithinRadius (LPtr p)
{
    int x = argInt(p);
    int y = argInt(p);
    int radius = argInt(p);
    return listFromIdArray(EnumeratePlanetsWithin(x, y, radius));
}

// (ships-at-planet p:pid)
static LPtr
FShipsAtPlanet (LPtr p1)
{
    int p = argPid(p1);
    return listFromIdArray(EnumerateShipsAtPlanet(p));
}

// (mines-covering x:int y:int)
static LPtr
FMinesCovering (LPtr p)
{
    int x = argInt(p);
    int y = argInt(p);
    return listFromIdArray(EnumerateMinesCovering(x, y));
}

// (mines-overlapping mfid:mfid)
static LPtr
FMinesOverlapping (LPtr p)
{
    int mfid = argMfid(p);
    return listFromIdArray(EnumerateOverlappingMines(mfid));
}

// (player-active-p p : player)
static LPtr
FPlayerActiveP (LPtr p1)
{
    RaceType_Def p = argPlayer(p1);
    return retBool(PlayerIsActive(p));
}

// (put-player-active-p p : player b : bool)
static LPtr
FPutPlayerActiveP (LPtr p1)
{
    RaceType_Def p = argPlayer(p1);
    bool b = argBool(p1);
    PutPlayerIsActive(p, Boolean(b));
    return LPtr();
}

// (player-password p : player)
static LPtr
FPlayerPassword (LPtr p1)
{
    RaceType_Def p = argPlayer(p1);
    return new LString(PlayerPassword(p));
}

// (put-player-password p : player pass : string)
static LPtr
FPutPlayerPassword (LPtr p1)
{
    RaceType_Def p = argPlayer(p1);
    std::string pass = argString(p1);
    PutPlayerPassword(p, pass.c_str());
    return LPtr();
}

// (kill-config what : int<0..5>)
static LPtr
FKillConfig (LPtr p)
{
    int what = argInt(p);
    if (what < 0 || what > 5)
        throw LLispException("range-error", "parameter what out of range 0..5");
    return retBool(KillPlayerConfigure(PlayerKillType_Def(what)));
}

// (put-kill-config what : int<0..5> ena:bool)
static LPtr
FPutKillConfig (LPtr p)
{
    int what = argInt(p);
    bool ena = argBool(p);
    if (what < 0 || what > 5)
        throw LLispException("range-error", "parameter what out of range 0..5");
    PutKillPlayerConfigure(PlayerKillType_Def(what), Boolean(ena));
    return LPtr();
}

// (kill-player p:player)
static LPtr
FKillPlayer (LPtr p1)
{
    RaceType_Def p = argPlayer(p1);
    KillPlayer(p);
    return LPtr();
}

// (kill-player-planets p:player)
static LPtr
FKillPlayerPlanets (LPtr p1)
{
    RaceType_Def p = argPlayer(p1);
    KillPlayerPlanets(p);
    return LPtr();
}

// (kill-player-bases p:player)
static LPtr
FKillPlayerBases (LPtr p1)
{
    RaceType_Def p = argPlayer(p1);
    KillPlayerBases(p);
    return LPtr();
}

// (kill-player-minefields p:player)
static LPtr
FKillPlayerMinefields (LPtr p1)
{
    RaceType_Def p = argPlayer(p1);
    KillPlayerMinefields(p);
    return LPtr();
}

// (kill-player-ships p:player)
static LPtr
FKillPlayerShips (LPtr p1)
{
    RaceType_Def p = argPlayer(p1);
    KillPlayerShips(p);
    return LPtr();
}

// (eff-race p : player)
static LPtr
FEffRace (LPtr p1)
{
    RaceType_Def p = argPlayer(p1);
    return new LInt(EffRace(p));
}

// (eff-mission p : player)
static LPtr
FEffMission (LPtr p1)
{
    RaceType_Def p = argPlayer(p1);
    return new LInt(EffMission(p));
}

// (build-queue-invalidate b:pid)
static LPtr
FBuildQueueInvalidate (LPtr p)
{
    int b = argPid(p);
    BuildQueueInvalidate(b);
    return LPtr();
}

// (ships-in-build-queue )
static LPtr
FShipsInBuildQueue (LPtr )
{
    return new LInt(ShipsInBuildQueue());
}

// (sort-build-queue )
static LPtr
FSortBuildQueue (LPtr )
{
    SortBuildQueue();
    return LPtr();
}

// (build-queue-base-id pos:int)
static LPtr
FBuildQueueBaseId (LPtr p)
{
    int pos = argInt(p);
    return new LInt(BuildQueueBaseID(pos));
}

// (build-queue-priority pos:int)
static LPtr
FBuildQueuePriority (LPtr p)
{
    int pos = argInt(p);
    return new LInt(BuildQueuePriority(pos));
}

// (build-queue-cloning-p pos:int)
static LPtr
FBuildQueueCloningP (LPtr p)
{
    int pos = argInt(p);
    return retBool(BuildQueueIsCloning(pos));
}

// (build-queue-ship-owner pos:int)
static LPtr
FBuildQueueShipOwner (LPtr p)
{
    int pos = argInt(p);
    return new LInt(BuildQueueShipOwner(pos));
}

// (build-queue-owner pos:int)
static LPtr
FBuildQueueOwner (LPtr p)
{
    int pos = argInt(p);
    return new LInt(BuildQueueOwner(pos));
}

// (build-queue-hull-type pos:int)
static LPtr
FBuildQueueHullType (LPtr p)
{
    int pos = argInt(p);
    return new LInt(BuildQueueHullType(pos));
}

// (build-queue-engine-type pos:int)
static LPtr
FBuildQueueEngineType (LPtr p)
{
    int pos = argInt(p);
    return new LInt(BuildQueueEngineType(pos));
}

// (build-queue-beam-type pos:int)
static LPtr
FBuildQueueBeamType (LPtr p)
{
    int pos = argInt(p);
    return new LInt(BuildQueueBeamType(pos));
}

// (build-queue-torp-type pos:int)
static LPtr
FBuildQueueTorpType (LPtr p)
{
    int pos = argInt(p);
    return new LInt(BuildQueueTorpType(pos));
}

// (build-queue-beam-number pos:int)
static LPtr
FBuildQueueBeamNumber (LPtr p)
{
    int pos = argInt(p);
    return new LInt(BuildQueueBeamNumber(pos));
}

// (build-queue-torp-number pos:int)
static LPtr
FBuildQueueTorpNumber (LPtr p)
{
    int pos = argInt(p);
    return new LInt(BuildQueueTorpNumber(pos));
}

// (put-build-queue-priority pos:int pri:int)
static LPtr
FPutBuildQueuePriority (LPtr p)
{
    int pos = argInt(p);
    int pri = argInt(p);
    PutBuildQueuePriority(pos, pri);
    return LPtr();
}

// (activity-level p:player)
static LPtr
FActivityLevel (LPtr p1)
{
    RaceType_Def p = argPlayer(p1);
    return new LInt(GetActivityLevel(p));
}

// (put-activity-level p:player val:int)
static LPtr
FPutActivityLevel (LPtr p1)
{
    RaceType_Def p = argPlayer(p1);
    int val = argInt(p1);
    SetActivityLevel(p, val);
    return LPtr();
}

// (ally-add p:player a:player)
static LPtr
FAllyAdd (LPtr p1)
{
    RaceType_Def p = argPlayer(p1);
    RaceType_Def a = argPlayer(p1);
    AllyAddRequest(p, a);
    return LPtr();
}

// (ally-drop p:player a:player)
static LPtr
FAllyDrop (LPtr p1)
{
    RaceType_Def p = argPlayer(p1);
    RaceType_Def a = argPlayer(p1);
    AllyDropRequest(p, a);
    return LPtr();
}

// (ally-allows-p p:player a:player lev:int<0..4>)
static LPtr
FAllyAllowsP (LPtr p1)
{
    RaceType_Def p = argPlayer(p1);
    RaceType_Def a = argPlayer(p1);
    int lev = argInt(p1);
    if (lev < 0 || lev > 4)
        throw LLispException("range-error", "parameter lev out of range 0..4");
    return retBool(PlayerAllowsAlly(p, a, AllianceLevel_Def(lev)));;
}

// (put-ally-allows p:player a:player lev:int<0..4> state:int<0..2>)
static LPtr
FPutAllyAllows (LPtr p1)
{
    RaceType_Def p = argPlayer(p1);
    RaceType_Def a = argPlayer(p1);
    int lev = argInt(p1);
    int state = argInt(p1);
    if (lev < 0 || lev > 4)
        throw LLispException("range-error", "parameter lev out of range 0..4");
    if (state < 0 || state > 2)
        throw LLispException("range-error", "parameter state out of range 0..2");
    PutPlayerAllowsAlly(p, a, AllianceLevel_Def(lev), AllianceState_Def(state));;
    return LPtr();
}

// (ally-offers-p p:player a:player)
static LPtr
FAllyOffersP (LPtr p1)
{
    RaceType_Def p = argPlayer(p1);
    RaceType_Def a = argPlayer(p1);
    return retBool(PlayerOffersAlliance(p, a));
}

// (players-are-allied-p p:player a:player)
static LPtr
FPlayersAreAlliedP (LPtr p1)
{
    RaceType_Def p = argPlayer(p1);
    RaceType_Def a = argPlayer(p1);
    return retBool(PlayersAreAllies(p, a));
}

// (ally-offer-conditional-p p:player a:player lev:int<0..4>)
static LPtr
FAllyOfferConditionalP (LPtr p1)
{
    RaceType_Def p = argPlayer(p1);
    RaceType_Def a = argPlayer(p1);
    int lev = argInt(p1);
    if (lev < 0 || lev > 4)
        throw LLispException("range-error", "parameter lev out of range 0..4");
    return retBool(IsAllyOfferConditional(p, a, AllianceLevel_Def(lev)));
}

// (ally-offer-any-p p:player a:player lev:int<0..4>)
static LPtr
FAllyOfferAnyP (LPtr p1)
{
    RaceType_Def p = argPlayer(p1);
    RaceType_Def a = argPlayer(p1);
    int lev = argInt(p1);
    if (lev < 0 || lev > 4)
        throw LLispException("range-error", "parameter lev out of range 0..4");
    return retBool(IsAllyOfferAny(p, a, AllianceLevel_Def(lev)));
}

// (ship-exists-p i:int)
static LPtr
FShipExistsP (LPtr p)
{
    int i = argInt(p);
    return retBool(IsShipExist(i));
}

// (ship-delete i:sid)
static LPtr
FShipDelete (LPtr p)
{
    int i = argSid(p);
    DeleteShip(i);
    return LPtr();
}

// (ship-owner i:sid)
static LPtr
FShipOwner (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipOwner(i));
}

// (ship-speed i:sid)
static LPtr
FShipSpeed (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipSpeed(i));
}

// (ship-location-x i:sid)
static LPtr
FShipLocationX (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipLocationX(i));
}

// (ship-location-y i:sid)
static LPtr
FShipLocationY (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipLocationY(i));
}

// (ship-waypoint-x i:sid)
static LPtr
FShipWaypointX (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipWaypointX(i));
}

// (ship-waypoint-y i:sid)
static LPtr
FShipWaypointY (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipWaypointY(i));
}

// (ship-waypoint-dx i:sid)
static LPtr
FShipWaypointDx (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipRelWaypointX(i));
}

// (ship-waypoint-dy i:sid)
static LPtr
FShipWaypointDy (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipRelWaypointY(i));
}

// (ship-hull i:sid)
static LPtr
FShipHull (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipHull(i));
}

// (ship-engine i:sid)
static LPtr
FShipEngine (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipEngine(i));
}

// (ship-beam-type i:sid)
static LPtr
FShipBeamType (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipBeamType(i));
}

// (ship-beam-number i:sid)
static LPtr
FShipBeamNumber (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipBeamNumber(i));
}

// (ship-torp-type i:sid)
static LPtr
FShipTorpType (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipTorpType(i));
}

// (ship-tube-number i:sid)
static LPtr
FShipTubeNumber (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipTubeNumber(i));
}

// (ship-bays i:sid)
static LPtr
FShipBays (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipBays(i));
}

// (ship-ammo i:sid)
static LPtr
FShipAmmo (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipAmmunition(i));
}

// (ship-mission i:sid)
static LPtr
FShipMission (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipMission(i));
}

// (ship-enemy i:sid)
static LPtr
FShipEnemy (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipEnemy(i));
}

// (ship-tow-target i:sid)
static LPtr
FShipTowTarget (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipTowTarget(i));
}

// (ship-intercept-target i:sid)
static LPtr
FShipInterceptTarget (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipInterceptTarget(i));
}

// (ship-damage i:sid)
static LPtr
FShipDamage (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipDamage(i));
}

// (ship-crew i:sid)
static LPtr
FShipCrew (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipCrew(i));
}

// (ship-unload-cargo i:sid typ:cargotype<0..6>)
static LPtr
FShipUnloadCargo (LPtr p)
{
    int i = argSid(p);
    int typ = argInt(p);
    if (typ < 0 || typ > 6)
        throw LLispException("range-error", "parameter typ out of range 0..6");
    return new LInt(ShipDump(i, CargoType_Def(typ)));
}

// (ship-unload-target i:sid)
static LPtr
FShipUnloadTarget (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipDumpPlanet(i));
}

// (ship-transfer-cargo i:sid typ:cargotype<0..6>)
static LPtr
FShipTransferCargo (LPtr p)
{
    int i = argSid(p);
    int typ = argInt(p);
    if (typ < 0 || typ > 6)
        throw LLispException("range-error", "parameter typ out of range 0..6");
    return new LInt(ShipTransfer(i, CargoType_Def(typ)));
}

// (ship-transfer-target i:sid)
static LPtr
FShipTransferTarget (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipTransferShip(i));
}

// (ship-name i:sid)
static LPtr
FShipName (LPtr p)
{
    int i = argSid(p);
    return new LString(ShipName(i, 0));
}

// (ship-fcode i:sid)
static LPtr
FShipFcode (LPtr p)
{
    int i = argSid(p);
    return new LString(ShipFC(i, 0));
}

// (ship-cargo i:sid typ:cargotype<0..6>)
static LPtr
FShipCargo (LPtr p)
{
    int i = argSid(p);
    int typ = argInt(p);
    if (typ < 0 || typ > 6)
        throw LLispException("range-error", "parameter typ out of range 0..6");
    return new LInt(ShipCargo(i, CargoType_Def(typ)));
}

// (make-ship p:player)
static LPtr
FMakeShip (LPtr p1)
{
    RaceType_Def p = argPlayer(p1);
    return intOrNull(CreateShip(p));
}

// (ship-mass i:sid)
static LPtr
FShipMass (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipTotalMass(i));
}

// (ship-engine-load i:sid)
static LPtr
FShipEngineLoad (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipTravelMass(i));
}

// (ship-cargo-mass i:sid)
static LPtr
FShipCargoMass (LPtr p)
{
    int i = argSid(p);
    return new LInt(ShipCargoMass(i));
}

// (ship-mission-str i:sid)
static LPtr
FShipMissionStr (LPtr p)
{
    int i = argSid(p);
    return new LString(ShipMissionString(i,0));
}

// (put-ship-cargo i:sid typ:cargotype<0..6> val:int<0..10000>)
static LPtr
FPutShipCargo (LPtr p)
{
    int i = argSid(p);
    int typ = argInt(p);
    int val = argInt(p);
    if (typ < 0 || typ > 6)
        throw LLispException("range-error", "parameter typ out of range 0..6");
    if (val < 0 || val > 10000)
        throw LLispException("range-error", "parameter val out of range 0..10000");
    PutShipCargo(i, CargoType_Def(typ), val);
    return LPtr();
}

// (put-ship-name i:sid name:string)
static LPtr
FPutShipName (LPtr p)
{
    int i = argSid(p);
    std::string name = argString(p);
    PutShipName(i, name.c_str());
    return LPtr();
}

// (put-ship-owner i:sid owner:int<1..12>)
static LPtr
FPutShipOwner (LPtr p)
{
    int i = argSid(p);
    int owner = argInt(p);
    if (owner < 1 || owner > 12)
        throw LLispException("range-error", "parameter owner out of range 1..12");
    PutShipOwner(i, RaceType_Def(owner));
    return LPtr();
}

// (put-ship-fcode i:sid fc:string)
static LPtr
FPutShipFcode (LPtr p)
{
    int i = argSid(p);
    std::string fc = argString(p);
    PutShipFCode(i, fc.c_str());
    return LPtr();
}

// (put-ship-speed i:sid sp:int<0..MAX_SPEED>)
static LPtr
FPutShipSpeed (LPtr p)
{
    int i = argSid(p);
    int sp = argInt(p);
    if (sp < 0 || sp > MAX_SPEED)
        throw LLispException("range-error", "parameter sp out of range 0..MAX_SPEED");
    PutShipSpeed(i, sp);
    return LPtr();
}

// (put-ship-waypoint-x i:sid x:int<0..MAX_COORDINATE>)
static LPtr
FPutShipWaypointX (LPtr p)
{
    int i = argSid(p);
    int x = argInt(p);
    if (x < 0 || x > MAX_COORDINATE)
        throw LLispException("range-error", "parameter x out of range 0..MAX_COORDINATE");
    PutShipWaypointX(i, x);
    return LPtr();
}

// (put-ship-waypoint-y i:sid y:int<0..MAX_COORDINATE>)
static LPtr
FPutShipWaypointY (LPtr p)
{
    int i = argSid(p);
    int y = argInt(p);
    if (y < 0 || y > MAX_COORDINATE)
        throw LLispException("range-error", "parameter y out of range 0..MAX_COORDINATE");
    PutShipWaypointY(i, y);
    return LPtr();
}

// (put-ship-waypoint-dx i:sid dx:int<-3000..3000>)
static LPtr
FPutShipWaypointDx (LPtr p)
{
    int i = argSid(p);
    int dx = argInt(p);
    if (dx < -3000 || dx > 3000)
        throw LLispException("range-error", "parameter dx out of range -3000..3000");
    PutShipRelWaypointX(i, dx);
    return LPtr();
}

// (put-ship-waypoint-dy i:sid dy:int<-3000..3000>)
static LPtr
FPutShipWaypointDy (LPtr p)
{
    int i = argSid(p);
    int dy = argInt(p);
    if (dy < -3000 || dy > 3000)
        throw LLispException("range-error", "parameter dy out of range -3000..3000");
    PutShipRelWaypointY(i, dy);
    return LPtr();
}

// (put-ship-location-x i:sid x:int<0..MAX_COORDINATE>)
static LPtr
FPutShipLocationX (LPtr p)
{
    int i = argSid(p);
    int x = argInt(p);
    if (x < 0 || x > MAX_COORDINATE)
        throw LLispException("range-error", "parameter x out of range 0..MAX_COORDINATE");
    PutShipLocationX(i, x);
    return LPtr();
}

// (put-ship-location-y i:sid y:int<0..MAX_COORDINATE>)
static LPtr
FPutShipLocationY (LPtr p)
{
    int i = argSid(p);
    int y = argInt(p);
    if (y < 0 || y > MAX_COORDINATE)
        throw LLispException("range-error", "parameter y out of range 0..MAX_COORDINATE");
    PutShipLocationY(i, y);
    return LPtr();
}

// (put-ship-hull i:sid h:int<1..HULL_NR>)
static LPtr
FPutShipHull (LPtr p)
{
    int i = argSid(p);
    int h = argInt(p);
    if (h < 1 || h > HULL_NR)
        throw LLispException("range-error", "parameter h out of range 1..HULL_NR");
    PutShipHull(i, h);
    return LPtr();
}

// (put-ship-engine i:sid h:int<1..ENGINE_NR>)
static LPtr
FPutShipEngine (LPtr p)
{
    int i = argSid(p);
    int h = argInt(p);
    if (h < 1 || h > ENGINE_NR)
        throw LLispException("range-error", "parameter h out of range 1..ENGINE_NR");
    PutShipEngine(i, h);
    return LPtr();
}

// (put-ship-beam-type i:sid h:int<0..BEAM_NR>)
static LPtr
FPutShipBeamType (LPtr p)
{
    int i = argSid(p);
    int h = argInt(p);
    if (h < 0 || h > BEAM_NR)
        throw LLispException("range-error", "parameter h out of range 0..BEAM_NR");
    PutShipBeamType(i, h);
    return LPtr();
}

// (put-ship-beam-number i:sid h:int<0..MAX_BEAMS>)
static LPtr
FPutShipBeamNumber (LPtr p)
{
    int i = argSid(p);
    int h = argInt(p);
    if (h < 0 || h > MAX_BEAMS)
        throw LLispException("range-error", "parameter h out of range 0..MAX_BEAMS");
    PutShipBeamNumber(i, h);
    return LPtr();
}

// (put-ship-torp-type i:sid t:int<0..TORP_NR>)
static LPtr
FPutShipTorpType (LPtr p)
{
    int i = argSid(p);
    int t = argInt(p);
    if (t < 0 || t > TORP_NR)
        throw LLispException("range-error", "parameter t out of range 0..TORP_NR");
    PutShipTorpType(i, t);
    return LPtr();
}

// (put-ship-tube-number i:sid t:int<0..MAX_TUBES>)
static LPtr
FPutShipTubeNumber (LPtr p)
{
    int i = argSid(p);
    int t = argInt(p);
    if (t < 0 || t > MAX_TUBES)
        throw LLispException("range-error", "parameter t out of range 0..MAX_TUBES");
    PutShipTubeNumber(i, t);
    return LPtr();
}

// (put-ship-bays i:sid t:int<0..MAX_BAYS>)
static LPtr
FPutShipBays (LPtr p)
{
    int i = argSid(p);
    int t = argInt(p);
    if (t < 0 || t > MAX_BAYS)
        throw LLispException("range-error", "parameter t out of range 0..MAX_BAYS");
    PutShipBays(i, t);
    return LPtr();
}

// (put-ship-ammo i:sid t:int<0..10000>)
static LPtr
FPutShipAmmo (LPtr p)
{
    int i = argSid(p);
    int t = argInt(p);
    if (t < 0 || t > 10000)
        throw LLispException("range-error", "parameter t out of range 0..10000");
    PutShipAmmunition(i, t);
    return LPtr();
}

// (put-ship-mission i:sid m:int<0..10000>)
static LPtr
FPutShipMission (LPtr p)
{
    int i = argSid(p);
    int m = argInt(p);
    if (m < 0 || m > 10000)
        throw LLispException("range-error", "parameter m out of range 0..10000");
    PutShipMission(i, ShipMission_Def(m));
    return LPtr();
}

// (put-ship-enemy i:sid m:int<0..11>)
static LPtr
FPutShipEnemy (LPtr p)
{
    int i = argSid(p);
    int m = argInt(p);
    if (m < 0 || m > 11)
        throw LLispException("range-error", "parameter m out of range 0..11");
    PutShipEnemy(i, RaceType_Def(m));
    return LPtr();
}

// (put-ship-tow-target i:sid a:int<0..10000>)
static LPtr
FPutShipTowTarget (LPtr p)
{
    int i = argSid(p);
    int a = argInt(p);
    if (a < 0 || a > 10000)
        throw LLispException("range-error", "parameter a out of range 0..10000");
    PutShipTowTarget(i, a);
    return LPtr();
}

// (put-ship-intercept-target i:sid a:int<0..10000>)
static LPtr
FPutShipInterceptTarget (LPtr p)
{
    int i = argSid(p);
    int a = argInt(p);
    if (a < 0 || a > 10000)
        throw LLispException("range-error", "parameter a out of range 0..10000");
    PutShipInterceptTarget(i, a);
    return LPtr();
}

// (put-ship-damage i:sid d:int<0..150>)
static LPtr
FPutShipDamage (LPtr p)
{
    int i = argSid(p);
    int d = argInt(p);
    if (d < 0 || d > 150)
        throw LLispException("range-error", "parameter d out of range 0..150");
    PutShipDamage(i, d);
    return LPtr();
}

// (put-ship-crew i:sid c:int<0..10000>)
static LPtr
FPutShipCrew (LPtr p)
{
    int i = argSid(p);
    int c = argInt(p);
    if (c < 0 || c > 10000)
        throw LLispException("range-error", "parameter c out of range 0..10000");
    PutShipCrew(i, c);
    return LPtr();
}

// (put-ship-unload-cargo i:sid typ:cargotype<0..6> n:int<0..10000>)
static LPtr
FPutShipUnloadCargo (LPtr p)
{
    int i = argSid(p);
    int typ = argInt(p);
    int n = argInt(p);
    if (typ < 0 || typ > 6)
        throw LLispException("range-error", "parameter typ out of range 0..6");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutShipDump(i, CargoType_Def(typ), n);
    return LPtr();
}

// (put-ship-unload-target i:sid n:int<0..PLANET_NR>)
static LPtr
FPutShipUnloadTarget (LPtr p)
{
    int i = argSid(p);
    int n = argInt(p);
    if (n < 0 || n > PLANET_NR)
        throw LLispException("range-error", "parameter n out of range 0..PLANET_NR");
    PutShipDumpPlanet(i, n);
    return LPtr();
}

// (put-ship-transfer-cargo i:sid typ:cargotype<0..6> n:int<0..10000>)
static LPtr
FPutShipTransferCargo (LPtr p)
{
    int i = argSid(p);
    int typ = argInt(p);
    int n = argInt(p);
    if (typ < 0 || typ > 6)
        throw LLispException("range-error", "parameter typ out of range 0..6");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutShipTransfer(i, CargoType_Def(typ), n);
    return LPtr();
}

// (put-ship-transfer-target i:sid n:int<0..SHIP_NR>)
static LPtr
FPutShipTransferTarget (LPtr p)
{
    int i = argSid(p);
    int n = argInt(p);
    if (n < 0 || n > SHIP_NR)
        throw LLispException("range-error", "parameter n out of range 0..SHIP_NR");
    PutShipTransferShip(i, n);
    return LPtr();
}

// (ship-cloaked-p i:sid)
static LPtr
FShipCloakedP (LPtr p)
{
    int i = argSid(p);
    return retBool(IsShipCloaked(i));
}

// (put-ship-cloaked-p i:sid cl:bool)
static LPtr
FPutShipCloakedP (LPtr p)
{
    int i = argSid(p);
    bool cl = argBool(p);
    MarkShipCloaking(i, Boolean(cl));
    return LPtr();
}

// (ship-shield-level i:int)
static LPtr
FShipShieldLevel (LPtr p)
{
    int i = argInt(p);
    return new LInt(ShipCombatShieldLevel(i));
}

// (put-ship-shield-level i:int sh:int<0..100>)
static LPtr
FPutShipShieldLevel (LPtr p)
{
    int i = argInt(p);
    int sh = argInt(p);
    if (sh < 0 || sh > 100)
        throw LLispException("range-error", "parameter sh out of range 0..100");
    SetShipCombatShieldLevel(i, sh);
    return LPtr();
}

// (planet-exists-p i:int)
static LPtr
FPlanetExistsP (LPtr p)
{
    int i = argInt(p);
    return retBool(IsPlanetExist(i));
}

// (planet-location-x i:pid)
static LPtr
FPlanetLocationX (LPtr p)
{
    int i = argPid(p);
    return new LInt(PlanetLocationX(i));
}

// (planet-location-y i:pid)
static LPtr
FPlanetLocationY (LPtr p)
{
    int i = argPid(p);
    return new LInt(PlanetLocationY(i));
}

// (planet-owner i:pid)
static LPtr
FPlanetOwner (LPtr p)
{
    int i = argPid(p);
    return new LInt(PlanetOwner(i));
}

// (planet-mines i:pid)
static LPtr
FPlanetMines (LPtr p)
{
    int i = argPid(p);
    return new LInt(PlanetMines(i));
}

// (planet-defense i:pid)
static LPtr
FPlanetDefense (LPtr p)
{
    int i = argPid(p);
    return new LInt(PlanetDefense(i));
}

// (planet-factories i:pid)
static LPtr
FPlanetFactories (LPtr p)
{
    int i = argPid(p);
    return new LInt(PlanetFactories(i));
}

// (planet-cargo i:pid typ:cargotype<0..CREDITS>)
static LPtr
FPlanetCargo (LPtr p)
{
    int i = argPid(p);
    int typ = argInt(p);
    if (typ < 0 || typ > CREDITS)
        throw LLispException("range-error", "parameter typ out of range 0..CREDITS");
    return new LInt(PlanetCargo(i, CargoType_Def(typ)));
}

// (planet-core i:pid typ:cargotype<0..3>)
static LPtr
FPlanetCore (LPtr p)
{
    int i = argPid(p);
    int typ = argInt(p);
    if (typ < 0 || typ > 3)
        throw LLispException("range-error", "parameter typ out of range 0..3");
    return new LInt(PlanetCore(i, CargoType_Def(typ)));
}

// (planet-density i:pid typ:cargotype<0..3>)
static LPtr
FPlanetDensity (LPtr p)
{
    int i = argPid(p);
    int typ = argInt(p);
    if (typ < 0 || typ > 3)
        throw LLispException("range-error", "parameter typ out of range 0..3");
    return new LInt(PlanetDensity(i, CargoType_Def(typ)));
}

// (planet-colonist-tax i:pid)
static LPtr
FPlanetColonistTax (LPtr p)
{
    int i = argPid(p);
    return new LInt(PlanetColTax(i));
}

// (planet-native-tax i:pid)
static LPtr
FPlanetNativeTax (LPtr p)
{
    int i = argPid(p);
    return new LInt(PlanetNatTax(i));
}

// (planet-colonist-happy i:pid)
static LPtr
FPlanetColonistHappy (LPtr p)
{
    int i = argPid(p);
    return new LInt(PlanetColHappy(i));
}

// (planet-native-happy i:pid)
static LPtr
FPlanetNativeHappy (LPtr p)
{
    int i = argPid(p);
    return new LInt(PlanetNatHappy(i));
}

// (planet-native-gov i:pid)
static LPtr
FPlanetNativeGov (LPtr p)
{
    int i = argPid(p);
    return new LInt(PlanetNatGovm(i));
}

// (planet-native-race i:pid)
static LPtr
FPlanetNativeRace (LPtr p)
{
    int i = argPid(p);
    return new LInt(PlanetNatType(i));
}

// (planet-native-pop i:pid)
static LPtr
FPlanetNativePop (LPtr p)
{
    int i = argPid(p);
    return new LInt(PlanetNativePopulation(i));
}

// (planet-temp i:pid)
static LPtr
FPlanetTemp (LPtr p)
{
    int i = argPid(p);
    return new LInt(PlanetTemp(i));
}

// (planet-build-base-p p:pid)
static LPtr
FPlanetBuildBaseP (LPtr p1)
{
    int p = argPid(p1);
    return retBool(PlanetBuildBase(p));
}

// (planet-has-natives-p p:pid)
static LPtr
FPlanetHasNativesP (LPtr p1)
{
    int p = argPid(p1);
    return retBool(PlanetHasNatives(p));
}

// (planet-native-race-str p:pid)
static LPtr
FPlanetNativeRaceStr (LPtr p1)
{
    int p = argPid(p1);
    return new LString(PlanetNatString(p, 0));
}

// (planet-native-gov-str p:pid)
static LPtr
FPlanetNativeGovStr (LPtr p1)
{
    int p = argPid(p1);
    return new LString(PlanetNatGovmString(p, 0));
}

// (planet-factories-max i:pid)
static LPtr
FPlanetFactoriesMax (LPtr p)
{
    int i = argPid(p);
    return new LInt(PlanetMaxFactories(i));
}

// (planet-defense-max i:pid)
static LPtr
FPlanetDefenseMax (LPtr p)
{
    int i = argPid(p);
    return new LInt(PlanetMaxDefense(i));
}

// (planet-mines-max i:pid)
static LPtr
FPlanetMinesMax (LPtr p)
{
    int i = argPid(p);
    return new LInt(PlanetMaxMines(i));
}

// (planet-mining i:pid min:int<0..3>)
static LPtr
FPlanetMining (LPtr p)
{
    int i = argPid(p);
    int min = argInt(p);
    if (min < 0 || min > 3)
        throw LLispException("range-error", "parameter min out of range 0..3");
    return new LInt(PlanetMining(i, min));
}

// (planet-native-gov-tax i:pid)
static LPtr
FPlanetNativeGovTax (LPtr p)
{
    int i = argPid(p);
    return new LInt(PlanetGovTaxRate(i));
}

// (planet-native-happy-change i:pid)
static LPtr
FPlanetNativeHappyChange (LPtr p)
{
    int i = argPid(p);
    return new LInt(PlanetNatHappyChange(i));
}

// (planet-colonist-happy-change i:pid)
static LPtr
FPlanetColonistHappyChange (LPtr p)
{
    int i = argPid(p);
    return new LInt(PlanetColHappyChange(i));
}

// (planet-native-income i:pid)
static LPtr
FPlanetNativeIncome (LPtr p)
{
    int i = argPid(p);
    return new LInt(PlanetNatIncome(i));
}

// (planet-colonist-income i:pid)
static LPtr
FPlanetColonistIncome (LPtr p)
{
    int i = argPid(p);
    return new LInt(PlanetColIncome(i));
}

// (planet-supply-production i:pid)
static LPtr
FPlanetSupplyProduction (LPtr p)
{
    int i = argPid(p);
    return new LInt(PlanetSuppIncome(i));
}

// (planet-num-ships-hissing i:pid)
static LPtr
FPlanetNumShipsHissing (LPtr p)
{
    int i = argPid(p);
    return new LInt(NumberOfShipsHissingPlanet(i));
}

// (planet-name i:pid)
static LPtr
FPlanetName (LPtr p)
{
    int i = argPid(p);
    return new LString(PlanetName(i, 0));
}

// (planet-fcode i:pid)
static LPtr
FPlanetFcode (LPtr p)
{
    int i = argPid(p);
    return new LString(PlanetFCode(i, 0));
}

// (planet-temp-str i:pid)
static LPtr
FPlanetTempStr (LPtr p)
{
    int i = argPid(p);
    return new LString(PlanetTempString(i));
}

// (put-planet-owner i:pid o:int<0..11>)
static LPtr
FPutPlanetOwner (LPtr p)
{
    int i = argPid(p);
    int o = argInt(p);
    if (o < 0 || o > 11)
        throw LLispException("range-error", "parameter o out of range 0..11");
    ChangePlanetOwner(i, RaceType_Def(o));
    return LPtr();
}

// (put-planet-owner-only i:pid o:int<0..11>)
static LPtr
FPutPlanetOwnerOnly (LPtr p)
{
    int i = argPid(p);
    int o = argInt(p);
    if (o < 0 || o > 11)
        throw LLispException("range-error", "parameter o out of range 0..11");
    PutPlanetOwner(i, RaceType_Def(o));
    return LPtr();
}

// (put-planet-fcode i:pid s:string)
static LPtr
FPutPlanetFcode (LPtr p)
{
    int i = argPid(p);
    std::string s = argString(p);
    PutPlanetFC(i, s.c_str());
    return LPtr();
}

// (put-planet-mines i:pid m:int<0..MAX_MINES>)
static LPtr
FPutPlanetMines (LPtr p)
{
    int i = argPid(p);
    int m = argInt(p);
    if (m < 0 || m > MAX_MINES)
        throw LLispException("range-error", "parameter m out of range 0..MAX_MINES");
    PutPlanetMines(i, m);
    return LPtr();
}

// (put-planet-factories i:pid m:int<0..MAX_FACTORIES>)
static LPtr
FPutPlanetFactories (LPtr p)
{
    int i = argPid(p);
    int m = argInt(p);
    if (m < 0 || m > MAX_FACTORIES)
        throw LLispException("range-error", "parameter m out of range 0..MAX_FACTORIES");
    PutPlanetFactories(i, m);
    return LPtr();
}

// (put-planet-defense i:pid m:int<0..MAX_DEFENSE>)
static LPtr
FPutPlanetDefense (LPtr p)
{
    int i = argPid(p);
    int m = argInt(p);
    if (m < 0 || m > MAX_DEFENSE)
        throw LLispException("range-error", "parameter m out of range 0..MAX_DEFENSE");
    PutPlanetDefense(i, m);
    return LPtr();
}

// (put-planet-cargo i:pid typ:cargotype<0..CREDITS> amt:int<0..100000000>)
static LPtr
FPutPlanetCargo (LPtr p)
{
    int i = argPid(p);
    int typ = argInt(p);
    int amt = argInt(p);
    if (typ < 0 || typ > CREDITS)
        throw LLispException("range-error", "parameter typ out of range 0..CREDITS");
    if (amt < 0 || amt > 100000000)
        throw LLispException("range-error", "parameter amt out of range 0..100000000");
    PutPlanetCargo(i, CargoType_Def(typ), amt);
    return LPtr();
}

// (put-planet-core i:pid typ:cargotype<0..3> amt:int<0..100000000>)
static LPtr
FPutPlanetCore (LPtr p)
{
    int i = argPid(p);
    int typ = argInt(p);
    int amt = argInt(p);
    if (typ < 0 || typ > 3)
        throw LLispException("range-error", "parameter typ out of range 0..3");
    if (amt < 0 || amt > 100000000)
        throw LLispException("range-error", "parameter amt out of range 0..100000000");
    PutPlanetCore(i, CargoType_Def(typ), amt);
    return LPtr();
}

// (put-planet-density i:pid typ:cargotype<0..3> amt:int<0..100>)
static LPtr
FPutPlanetDensity (LPtr p)
{
    int i = argPid(p);
    int typ = argInt(p);
    int amt = argInt(p);
    if (typ < 0 || typ > 3)
        throw LLispException("range-error", "parameter typ out of range 0..3");
    if (amt < 0 || amt > 100)
        throw LLispException("range-error", "parameter amt out of range 0..100");
    PutPlanetDensity(i, CargoType_Def(typ), amt);
    return LPtr();
}

// (put-planet-colonist-tax i:pid tax:int<0..MAX_TAX>)
static LPtr
FPutPlanetColonistTax (LPtr p)
{
    int i = argPid(p);
    int tax = argInt(p);
    if (tax < 0 || tax > MAX_TAX)
        throw LLispException("range-error", "parameter tax out of range 0..MAX_TAX");
    PutPlanetColTax(i, tax);
    return LPtr();
}

// (put-planet-native-tax i:pid tax:int<0..MAX_TAX>)
static LPtr
FPutPlanetNativeTax (LPtr p)
{
    int i = argPid(p);
    int tax = argInt(p);
    if (tax < 0 || tax > MAX_TAX)
        throw LLispException("range-error", "parameter tax out of range 0..MAX_TAX");
    PutPlanetNatTax(i, tax);
    return LPtr();
}

// (put-planet-colonist-happy i:pid h:int<MIN_HAPPY..MAX_HAPPY>)
static LPtr
FPutPlanetColonistHappy (LPtr p)
{
    int i = argPid(p);
    int h = argInt(p);
    if (h < MIN_HAPPY || h > MAX_HAPPY)
        throw LLispException("range-error", "parameter h out of range MIN_HAPPY..MAX_HAPPY");
    PutPlanetColHappy(i, h);
    return LPtr();
}

// (put-planet-native-happy i:pid h:int<MIN_HAPPY..MAX_HAPPY>)
static LPtr
FPutPlanetNativeHappy (LPtr p)
{
    int i = argPid(p);
    int h = argInt(p);
    if (h < MIN_HAPPY || h > MAX_HAPPY)
        throw LLispException("range-error", "parameter h out of range MIN_HAPPY..MAX_HAPPY");
    PutPlanetNatHappy(i, h);
    return LPtr();
}

// (put-planet-native-gov i:pid h:int<0..NumGovmTypes>)
static LPtr
FPutPlanetNativeGov (LPtr p)
{
    int i = argPid(p);
    int h = argInt(p);
    if (h < 0 || h > NumGovmTypes)
        throw LLispException("range-error", "parameter h out of range 0..NumGovmTypes");
    PutPlanetNatGovm(i, NativeGovm_Def(h));
    return LPtr();
}

// (put-planet-native-pop i:pid pop:int<0..MAX_POPULATION>)
static LPtr
FPutPlanetNativePop (LPtr p)
{
    int i = argPid(p);
    int pop = argInt(p);
    if (pop < 0 || pop > MAX_POPULATION)
        throw LLispException("range-error", "parameter pop out of range 0..MAX_POPULATION");
    PutPlanetNativePopulation(i, pop);
    return LPtr();
}

// (put-planet-native-race i:pid r:int<0..NumNativeTypes>)
static LPtr
FPutPlanetNativeRace (LPtr p)
{
    int i = argPid(p);
    int r = argInt(p);
    if (r < 0 || r > NumNativeTypes)
        throw LLispException("range-error", "parameter r out of range 0..NumNativeTypes");
    PutPlanetNatType(i, NativeType_Def(r));
    return LPtr();
}

// (put-planet-temp i:pid t:int<0..MAX_TEMP>)
static LPtr
FPutPlanetTemp (LPtr p)
{
    int i = argPid(p);
    int t = argInt(p);
    if (t < 0 || t > MAX_TEMP)
        throw LLispException("range-error", "parameter t out of range 0..MAX_TEMP");
    PutPlanetTemp(i, t);
    return LPtr();
}

// (put-planet-build-base-p i:pid b:bool)
static LPtr
FPutPlanetBuildBaseP (LPtr p)
{
    int i = argPid(p);
    bool b = argBool(p);
    PutPlanetBuildBase(i, Boolean(b));
    return LPtr();
}

// (put-planet-name i:pid n:string)
static LPtr
FPutPlanetName (LPtr p)
{
    int i = argPid(p);
    std::string n = argString(p);
    PutPlanetName(i, n.c_str());
    return LPtr();
}

// (put-planet-location-x i:pid x:int<0..MAX_COORDINATE>)
static LPtr
FPutPlanetLocationX (LPtr p)
{
    int i = argPid(p);
    int x = argInt(p);
    if (x < 0 || x > MAX_COORDINATE)
        throw LLispException("range-error", "parameter x out of range 0..MAX_COORDINATE");
    PutPlanetLocationX(i, x);
    return LPtr();
}

// (put-planet-location-y i:pid x:int<0..MAX_COORDINATE>)
static LPtr
FPutPlanetLocationY (LPtr p)
{
    int i = argPid(p);
    int x = argInt(p);
    if (x < 0 || x > MAX_COORDINATE)
        throw LLispException("range-error", "parameter x out of range 0..MAX_COORDINATE");
    PutPlanetLocationY(i, x);
    return LPtr();
}

// (player-long-name i:player)
static LPtr
FPlayerLongName (LPtr p)
{
    RaceType_Def i = argPlayer(p);
    return new LString(RaceLongName(i, 0));
}

// (player-short-name i:player)
static LPtr
FPlayerShortName (LPtr p)
{
    RaceType_Def i = argPlayer(p);
    return new LString(RaceShortName(i, 0));
}

// (player-adj-name i:player)
static LPtr
FPlayerAdjName (LPtr p)
{
    RaceType_Def i = argPlayer(p);
    return new LString(RaceNameAdjective(i, 0));
}

// (put-player-long-name i:player s:string)
static LPtr
FPutPlayerLongName (LPtr p)
{
    RaceType_Def i = argPlayer(p);
    std::string s = argString(p);
    PutRaceLongName(i, s.c_str());
    return LPtr();
}

// (put-player-short-name i:player s:string)
static LPtr
FPutPlayerShortName (LPtr p)
{
    RaceType_Def i = argPlayer(p);
    std::string s = argString(p);
    PutRaceShortName(i, s.c_str());
    return LPtr();
}

// (put-player-adj-name i:player s:string)
static LPtr
FPutPlayerAdjName (LPtr p)
{
    RaceType_Def i = argPlayer(p);
    std::string s = argString(p);
    PutRaceNameAdjective(i, s.c_str());
    return LPtr();
}

// (turn-time )
static LPtr
FTurnTime (LPtr )
{
    char tmp[20];;
    HostTimeStamp(tmp);;
    return new LString(std::string(tmp, tmp+18));;
}

// (turn-number )
static LPtr
FTurnNumber (LPtr )
{
    return new LInt(TurnNumber());
}

// (make-fcode-special s:string)
static LPtr
FMakeFcodeSpecial (LPtr p)
{
    std::string s = argString(p);
    DefineSpecialFCode(s.c_str());
    return LPtr();
}

// (fcode-special-p s:string)
static LPtr
FFcodeSpecialP (LPtr p)
{
    std::string s = argString(p);
    return retBool(IsFCodeSpecial(s.c_str()));
}

// (ship-fcode-special-p i:sid)
static LPtr
FShipFcodeSpecialP (LPtr p)
{
    int i = argSid(p);
    return retBool(IsShipFCodeSpecial(i));
}

// (planet-fcode-special-p i:pid)
static LPtr
FPlanetFcodeSpecialP (LPtr p)
{
    int i = argPid(p);
    return retBool(IsPlanetFCodeSpecial(i));
}

// (fcode-match-p a:string b:string)
static LPtr
FFcodeMatchP (LPtr p)
{
    std::string a = argString(p);
    std::string b = argString(p);
    return retBool(IsFCodeMatch(a.c_str(), b.c_str()));
}

// (tons-add w:player l:player tons:int)
static LPtr
FTonsAdd (LPtr p)
{
    RaceType_Def w = argPlayer(p);
    RaceType_Def l = argPlayer(p);
    int tons = argInt(p);
    AddToTonsScore(w, l, tons);
    return LPtr();
}

// (player-tons w:player which:int<0..3>)
static LPtr
FPlayerTons (LPtr p)
{
    RaceType_Def w = argPlayer(p);
    int which = argInt(p);
    if (which < 0 || which > 3)
        throw LLispException("range-error", "parameter which out of range 0..3");
    return new LInt(GetTons(Tons(which),w));
}

// (put-player-tons w:player which:int<0..3> val:int)
static LPtr
FPutPlayerTons (LPtr p)
{
    RaceType_Def w = argPlayer(p);
    int which = argInt(p);
    int val = argInt(p);
    if (which < 0 || which > 3)
        throw LLispException("range-error", "parameter which out of range 0..3");
    PutTons(Tons(which),w,val);
    return LPtr();
}

// (tons-clear-this-turn )
static LPtr
FTonsClearThisTurn (LPtr )
{
    ClearThisTurnTons();
    return LPtr();
}

// (tons-read )
static LPtr
FTonsRead (LPtr )
{
    ReadTonsFile();
    return LPtr();
}

// (tons-write )
static LPtr
FTonsWrite (LPtr )
{
    WriteTonsFile();
    return LPtr();
}

// (ship-has-enemy s:sid pl:player)
static LPtr
FShipHasEnemy (LPtr p)
{
    int s = argSid(p);
    RaceType_Def pl = argPlayer(p);
    return retBool(ShipHasEnemy(s, pl));
}

// (player-has-enemy p:player e:player)
static LPtr
FPlayerHasEnemy (LPtr p1)
{
    RaceType_Def p = argPlayer(p1);
    RaceType_Def e = argPlayer(p1);
    return retBool(PlayerHasEnemy(p, e));
}

// (put-player-has-enemy p:player e:player v:bool)
static LPtr
FPutPlayerHasEnemy (LPtr p1)
{
    RaceType_Def p = argPlayer(p1);
    RaceType_Def e = argPlayer(p1);
    bool v = argBool(p1);
    return retBool(PutPlayerHasEnemy(p, e, Boolean(v)));
}

// (player-enemy-supported-p )
static LPtr
FPlayerEnemySupportedP (LPtr )
{
    return retBool(IsPlayerEnemyAllowed());
}

// (distance-squared x1:int y1:int x2:int y2:int)
static LPtr
FDistanceSquared (LPtr p)
{
    int x1 = argInt(p);
    int y1 = argInt(p);
    int x2 = argInt(p);
    int y2 = argInt(p);
    return new LInt(DistanceSquared(x1, y1, x2, y2));
}

// (distance x1:int y1:int x2:int y2:int)
static LPtr
FDistance (LPtr p)
{
    int x1 = argInt(p);
    int y1 = argInt(p);
    int x2 = argInt(p);
    int y2 = argInt(p);
    return new LInt(int(Distance(x1, y1, x2, y2)));
}

// (distance-lt-radius-p x1:int y1:int x2:int y2:int r:int)
static LPtr
FDistanceLtRadiusP (LPtr p)
{
    int x1 = argInt(p);
    int y1 = argInt(p);
    int x2 = argInt(p);
    int y2 = argInt(p);
    int r = argInt(p);
    return retBool(IsDistanceLTRadius(x1, y1, x2, y2, r));
}

// (wrap-x x:int)
static LPtr
FWrapX (LPtr p)
{
    int x = argInt(p);
    return new LInt(WrapMapX(x));
}

// (wrap-y y:int)
static LPtr
FWrapY (LPtr p)
{
    int y = argInt(p);
    return new LInt(WrapMapY(y));
}

// (wrap-dx dx:int)
static LPtr
FWrapDx (LPtr p)
{
    int dx = argInt(p);
    return new LInt(WrapDistX(dx));
}

// (wrap-dy dy:int)
static LPtr
FWrapDy (LPtr p)
{
    int dy = argInt(p);
    return new LInt(WrapDistY(dy));
}

// (point-on-map-p x:int y:int)
static LPtr
FPointOnMapP (LPtr p)
{
    int x = argInt(p);
    int y = argInt(p);
    return retBool(IsPointOnWrapMap(x, y));
}

// (rewrap-ships-and-mines )
static LPtr
FRewrapShipsAndMines (LPtr )
{
    RewrapShipsAndMines();
    return LPtr();
}

// (num-minefields )
static LPtr
FNumMinefields (LPtr )
{
    return new LInt(GetNumMinefields());
}

// (minefield-delete i:int)
static LPtr
FMinefieldDelete (LPtr p)
{
    int i = argInt(p);
    PutMinefieldUnits(i, 0);
    return LPtr();
}

// (minefield-exists-p i:int)
static LPtr
FMinefieldExistsP (LPtr p)
{
    int i = argInt(p);
    return retBool(IsMinefieldExist(i));
}

// (minefield-radius i:mfid)
static LPtr
FMinefieldRadius (LPtr p)
{
    int i = argMfid(p);
    return new LInt(int(MinefieldRadius(i)));
}

// (minefield-location-x i:mfid)
static LPtr
FMinefieldLocationX (LPtr p)
{
    int i = argMfid(p);
    return new LInt(MinefieldPositionX(i));
}

// (minefield-location-y i:mfid)
static LPtr
FMinefieldLocationY (LPtr p)
{
    int i = argMfid(p);
    return new LInt(MinefieldPositionY(i));
}

// (minefield-owner i:mfid)
static LPtr
FMinefieldOwner (LPtr p)
{
    int i = argMfid(p);
    return new LInt(MinefieldOwner(i));
}

// (minefield-units i:mfid)
static LPtr
FMinefieldUnits (LPtr p)
{
    int i = argMfid(p);
    return new LInt(MinefieldUnits(i));
}

// (minefield-web-p i:mfid)
static LPtr
FMinefieldWebP (LPtr p)
{
    int i = argMfid(p);
    return retBool(IsMinefieldWeb(i));
}

// (put-minefield-units i:mfid u:int<0..25000000>)
static LPtr
FPutMinefieldUnits (LPtr p)
{
    int i = argMfid(p);
    int u = argInt(p);
    if (u < 0 || u > 25000000)
        throw LLispException("range-error", "parameter u out of range 0..25000000");
    PutMinefieldUnits(i, u);
    return LPtr();
}

// (put-minefield-owner i:mfid r:player)
static LPtr
FPutMinefieldOwner (LPtr p)
{
    int i = argMfid(p);
    RaceType_Def r = argPlayer(p);
    PutMinefieldOwner(i, r);
    return LPtr();
}

// (put-minefield-location-x i:mfid x:int<0..MAX_COORDINATE>)
static LPtr
FPutMinefieldLocationX (LPtr p)
{
    int i = argMfid(p);
    int x = argInt(p);
    if (x < 0 || x > MAX_COORDINATE)
        throw LLispException("range-error", "parameter x out of range 0..MAX_COORDINATE");
    PutMinefieldPositionX(i, x);
    return LPtr();
}

// (put-minefield-location-y i:mfid y:int<0..MAX_COORDINATE>)
static LPtr
FPutMinefieldLocationY (LPtr p)
{
    int i = argMfid(p);
    int y = argInt(p);
    if (y < 0 || y > MAX_COORDINATE)
        throw LLispException("range-error", "parameter y out of range 0..MAX_COORDINATE");
    PutMinefieldPositionY(i, y);
    return LPtr();
}

// (put-minefield-web-p i:mfid web:bool)
static LPtr
FPutMinefieldWebP (LPtr p)
{
    int i = argMfid(p);
    bool web = argBool(p);
    PutMinefieldWeb(i, Boolean(web));
    return LPtr();
}

// (make-minefield x:int<0..MAX_COORDINATE> y:int<0..MAX_COORDINATE> p:player u:int<0..25000000> web:bool)
static LPtr
FMakeMinefield (LPtr p1)
{
    int x = argInt(p1);
    int y = argInt(p1);
    RaceType_Def p = argPlayer(p1);
    int u = argInt(p1);
    bool web = argBool(p1);
    if (x < 0 || x > MAX_COORDINATE)
        throw LLispException("range-error", "parameter x out of range 0..MAX_COORDINATE");
    if (y < 0 || y > MAX_COORDINATE)
        throw LLispException("range-error", "parameter y out of range 0..MAX_COORDINATE");
    if (u < 0 || u > 25000000)
        throw LLispException("range-error", "parameter u out of range 0..25000000");
    return intOrNull(CreateMinefield(x, y, p, u, Boolean(web)));
}

// (base-delete i:bid)
static LPtr
FBaseDelete (LPtr p)
{
    int i = argBid(p);
    DeleteBase(i);
    return LPtr();
}

// (base-exists-p i:int)
static LPtr
FBaseExistsP (LPtr p)
{
    int i = argInt(p);
    return retBool(IsBaseExist(i));
}

// (base-present-p i:int)
static LPtr
FBasePresentP (LPtr p)
{
    int i = argInt(p);
    return retBool(IsBasePresent(i));
}

// (base-owner i:bid)
static LPtr
FBaseOwner (LPtr p)
{
    int i = argBid(p);
    return new LInt(BaseOwner(i));
}

// (base-defense i:bid)
static LPtr
FBaseDefense (LPtr p)
{
    int i = argBid(p);
    return new LInt(BaseDefense(i));
}

// (base-damage i:bid)
static LPtr
FBaseDamage (LPtr p)
{
    int i = argBid(p);
    return new LInt(BaseDamage(i));
}

// (base-mission i:bid)
static LPtr
FBaseMission (LPtr p)
{
    int i = argBid(p);
    return new LInt(BaseOrder(i));
}

// (base-fix-order i:bid)
static LPtr
FBaseFixOrder (LPtr p)
{
    int i = argBid(p);
    return new LInt(BaseFixOrder(i));
}

// (base-fix-ship i:bid)
static LPtr
FBaseFixShip (LPtr p)
{
    int i = argBid(p);
    return new LInt(BaseFixingShip(i));
}

// (base-fighters i:bid)
static LPtr
FBaseFighters (LPtr p)
{
    int i = argBid(p);
    return new LInt(BaseFighters(i));
}

// (base-tech i:bid lev:int<0..3>)
static LPtr
FBaseTech (LPtr p)
{
    int i = argBid(p);
    int lev = argInt(p);
    if (lev < 0 || lev > 3)
        throw LLispException("range-error", "parameter lev out of range 0..3");
    return new LInt(BaseTech(i, BaseTech_Def(lev)));
}

// (base-hulls i:bid typ:int<1..20>)
static LPtr
FBaseHulls (LPtr p)
{
    int i = argBid(p);
    int typ = argInt(p);
    if (typ < 1 || typ > 20)
        throw LLispException("range-error", "parameter typ out of range 1..20");
    return new LInt(BaseHulls(i, typ));
}

// (base-engines i:bid typ:int<1..ENGINE_NR>)
static LPtr
FBaseEngines (LPtr p)
{
    int i = argBid(p);
    int typ = argInt(p);
    if (typ < 1 || typ > ENGINE_NR)
        throw LLispException("range-error", "parameter typ out of range 1..ENGINE_NR");
    return new LInt(BaseEngines(i, typ));
}

// (base-beams i:bid typ:int<1..BEAM_NR>)
static LPtr
FBaseBeams (LPtr p)
{
    int i = argBid(p);
    int typ = argInt(p);
    if (typ < 1 || typ > BEAM_NR)
        throw LLispException("range-error", "parameter typ out of range 1..BEAM_NR");
    return new LInt(BaseBeams(i, typ));
}

// (base-tubes i:bid typ:int<1..TORP_NR>)
static LPtr
FBaseTubes (LPtr p)
{
    int i = argBid(p);
    int typ = argInt(p);
    if (typ < 1 || typ > TORP_NR)
        throw LLispException("range-error", "parameter typ out of range 1..TORP_NR");
    return new LInt(BaseTubes(i, typ));
}

// (base-torps i:bid typ:int<1..TORP_NR>)
static LPtr
FBaseTorps (LPtr p)
{
    int i = argBid(p);
    int typ = argInt(p);
    if (typ < 1 || typ > TORP_NR)
        throw LLispException("range-error", "parameter typ out of range 1..TORP_NR");
    return new LInt(BaseTorps(i, typ));
}

// (base-build-order i:bid)
static LPtr
FBaseBuildOrder (LPtr p)
{
    int i = argBid(p);
    BuildOrder_Struct order;
    if (BaseBuildOrder(i, &order)) {
      return new LCons(new LInt(order.mHull),
             new LCons(new LInt(order.mEngineType),
             new LCons(new LInt(order.mBeamType),
             new LCons(new LInt(order.mNumBeams),
             new LCons(new LInt(order.mTubeType),
             new LCons(new LInt(order.mNumTubes), LPtr()))))));
    } else {
      return LPtr();
    }
}

// (make-base i:pid)
static LPtr
FMakeBase (LPtr p)
{
    int i = argPid(p);
    CreateBase(i);
    return LPtr();
}

// (put-base-owner i:bid p:player)
static LPtr
FPutBaseOwner (LPtr p1)
{
    int i = argBid(p1);
    RaceType_Def p = argPlayer(p1);
    PutBaseOwner(i, p);
    return LPtr();
}

// (put-base-defense i:bid n:int<0..MAX_BASE_DEFENSE>)
static LPtr
FPutBaseDefense (LPtr p)
{
    int i = argBid(p);
    int n = argInt(p);
    if (n < 0 || n > MAX_BASE_DEFENSE)
        throw LLispException("range-error", "parameter n out of range 0..MAX_BASE_DEFENSE");
    PutBaseDefense(i, n);
    return LPtr();
}

// (put-base-damage i:bid n:int<0..100>)
static LPtr
FPutBaseDamage (LPtr p)
{
    int i = argBid(p);
    int n = argInt(p);
    if (n < 0 || n > 100)
        throw LLispException("range-error", "parameter n out of range 0..100");
    PutBaseDamage(i, n);
    return LPtr();
}

// (put-base-fighters i:bid n:int<0..MAX_BASE_FIGHTERS>)
static LPtr
FPutBaseFighters (LPtr p)
{
    int i = argBid(p);
    int n = argInt(p);
    if (n < 0 || n > MAX_BASE_FIGHTERS)
        throw LLispException("range-error", "parameter n out of range 0..MAX_BASE_FIGHTERS");
    PutBaseFighters(i, n);
    return LPtr();
}

// (put-base-mission i:bid n:int<0..6>)
static LPtr
FPutBaseMission (LPtr p)
{
    int i = argBid(p);
    int n = argInt(p);
    if (n < 0 || n > 6)
        throw LLispException("range-error", "parameter n out of range 0..6");
    PutBaseOrder(i, BaseMission_Def(n));
    return LPtr();
}

// (put-base-fix-order i:bid n:int<0..2>)
static LPtr
FPutBaseFixOrder (LPtr p)
{
    int i = argBid(p);
    int n = argInt(p);
    if (n < 0 || n > 2)
        throw LLispException("range-error", "parameter n out of range 0..2");
    PutBaseFixOrder(i, BaseFixMission_Def(n));
    return LPtr();
}

// (put-base-fix-ship i:bid s:int<0..SHIP_NR>)
static LPtr
FPutBaseFixShip (LPtr p)
{
    int i = argBid(p);
    int s = argInt(p);
    if (s < 0 || s > SHIP_NR)
        throw LLispException("range-error", "parameter s out of range 0..SHIP_NR");
    PutBaseFixingShip(i, s);
    return LPtr();
}

// (put-base-tech i:bid lev:int<0..3> tech:int<1..MAX_TECH>)
static LPtr
FPutBaseTech (LPtr p)
{
    int i = argBid(p);
    int lev = argInt(p);
    int tech = argInt(p);
    if (lev < 0 || lev > 3)
        throw LLispException("range-error", "parameter lev out of range 0..3");
    if (tech < 1 || tech > MAX_TECH)
        throw LLispException("range-error", "parameter tech out of range 1..MAX_TECH");
    PutBaseTech(i, BaseTech_Def(lev), tech);
    return LPtr();
}

// (put-base-hulls i:bid typ:int<1..20> num:int<0..10000>)
static LPtr
FPutBaseHulls (LPtr p)
{
    int i = argBid(p);
    int typ = argInt(p);
    int num = argInt(p);
    if (typ < 1 || typ > 20)
        throw LLispException("range-error", "parameter typ out of range 1..20");
    if (num < 0 || num > 10000)
        throw LLispException("range-error", "parameter num out of range 0..10000");
    PutBaseHulls(i, typ, num);
    return LPtr();
}

// (put-base-engines i:bid typ:int<1..ENGINE_NR> num:int<0..10000>)
static LPtr
FPutBaseEngines (LPtr p)
{
    int i = argBid(p);
    int typ = argInt(p);
    int num = argInt(p);
    if (typ < 1 || typ > ENGINE_NR)
        throw LLispException("range-error", "parameter typ out of range 1..ENGINE_NR");
    if (num < 0 || num > 10000)
        throw LLispException("range-error", "parameter num out of range 0..10000");
    PutBaseEngines(i, typ, num);
    return LPtr();
}

// (put-base-beams i:bid typ:int<1..BEAM_NR> num:int<0..10000>)
static LPtr
FPutBaseBeams (LPtr p)
{
    int i = argBid(p);
    int typ = argInt(p);
    int num = argInt(p);
    if (typ < 1 || typ > BEAM_NR)
        throw LLispException("range-error", "parameter typ out of range 1..BEAM_NR");
    if (num < 0 || num > 10000)
        throw LLispException("range-error", "parameter num out of range 0..10000");
    PutBaseBeams(i, typ, num);
    return LPtr();
}

// (put-base-tubes i:bid typ:int<1..TORP_NR> num:int<0..10000>)
static LPtr
FPutBaseTubes (LPtr p)
{
    int i = argBid(p);
    int typ = argInt(p);
    int num = argInt(p);
    if (typ < 1 || typ > TORP_NR)
        throw LLispException("range-error", "parameter typ out of range 1..TORP_NR");
    if (num < 0 || num > 10000)
        throw LLispException("range-error", "parameter num out of range 0..10000");
    PutBaseTubes(i, typ, num);
    return LPtr();
}

// (put-base-torps i:bid typ:int<1..TORP_NR> num:int<0..10000>)
static LPtr
FPutBaseTorps (LPtr p)
{
    int i = argBid(p);
    int typ = argInt(p);
    int num = argInt(p);
    if (typ < 1 || typ > TORP_NR)
        throw LLispException("range-error", "parameter typ out of range 1..TORP_NR");
    if (num < 0 || num > 10000)
        throw LLispException("range-error", "parameter num out of range 0..10000");
    PutBaseTorps(i, typ, num);
    return LPtr();
}

// (put-base-build-order i:bid o:any)
static LPtr
FPutBaseBuildOrder (LPtr p)
{
    int i = argBid(p);
    LPtr o = getArg(p);
    BuildOrder_Struct order;
    memset(&order, 0, sizeof(order));
    if (o.nonnull())
      order.mHull = argInt(o);
    if (o.nonnull())
      order.mEngineType = argInt(o);
    if (o.nonnull())
      order.mBeamType = argInt(o);
    if (o.nonnull())
      order.mNumBeams = argInt(o);
    if (o.nonnull())
      order.mTubeType = argInt(o);
    if (o.nonnull())
      order.mNumTubes = argInt(o);
    if (o.nonnull())
      throw LLispException("range-error", "too many elements for build order");
    if (order.mHull > 20)
      throw LLispException("range-error", "hull type out of range");
    // FIXME: we would need many more checks
    PutBaseBuildOrder(i, &order);
    return LPtr();
}

// (base-clear-build-order i:bid)
static LPtr
FBaseClearBuildOrder (LPtr p)
{
    int i = argBid(p);
    ClearBaseBuildOrder(i);
    return LPtr();
}

// (base-clear-hulls i:bid)
static LPtr
FBaseClearHulls (LPtr p)
{
    int i = argBid(p);
    ClearBaseHulls(i);
    return LPtr();
}

// (engine-name i:int<1..ENGINE_NR>)
static LPtr
FEngineName (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > ENGINE_NR)
        throw LLispException("range-error", "parameter i out of range 1..ENGINE_NR");
    return new LString(EngineName(i, 0));
}

// (engine-cost-mc i:int<1..ENGINE_NR>)
static LPtr
FEngineCostMc (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > ENGINE_NR)
        throw LLispException("range-error", "parameter i out of range 1..ENGINE_NR");
    return new LInt(EngMoneyCost(i));
}

// (engine-cost-t i:int<1..ENGINE_NR>)
static LPtr
FEngineCostT (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > ENGINE_NR)
        throw LLispException("range-error", "parameter i out of range 1..ENGINE_NR");
    return new LInt(EngTritCost(i));
}

// (engine-cost-d i:int<1..ENGINE_NR>)
static LPtr
FEngineCostD (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > ENGINE_NR)
        throw LLispException("range-error", "parameter i out of range 1..ENGINE_NR");
    return new LInt(EngDurCost(i));
}

// (engine-cost-m i:int<1..ENGINE_NR>)
static LPtr
FEngineCostM (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > ENGINE_NR)
        throw LLispException("range-error", "parameter i out of range 1..ENGINE_NR");
    return new LInt(EngMolyCost(i));
}

// (engine-tech i:int<1..ENGINE_NR>)
static LPtr
FEngineTech (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > ENGINE_NR)
        throw LLispException("range-error", "parameter i out of range 1..ENGINE_NR");
    return new LInt(EngTechLevel(i));
}

// (engine-fuel-usage i:int<1..ENGINE_NR> sp:int<0..MAX_SPEED>)
static LPtr
FEngineFuelUsage (LPtr p)
{
    int i = argInt(p);
    int sp = argInt(p);
    if (i < 1 || i > ENGINE_NR)
        throw LLispException("range-error", "parameter i out of range 1..ENGINE_NR");
    if (sp < 0 || sp > MAX_SPEED)
        throw LLispException("range-error", "parameter sp out of range 0..MAX_SPEED");
    return new LInt(EngFuelConsumption(i, sp));
}

// (put-engine-name i:int<1..ENGINE_NR> n:string)
static LPtr
FPutEngineName (LPtr p)
{
    int i = argInt(p);
    std::string n = argString(p);
    if (i < 1 || i > ENGINE_NR)
        throw LLispException("range-error", "parameter i out of range 1..ENGINE_NR");
    PutEngineName(i, n.c_str());
    return LPtr();
}

// (put-engine-cost-mc i:int<1..ENGINE_NR> n:int<0..10000>)
static LPtr
FPutEngineCostMc (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > ENGINE_NR)
        throw LLispException("range-error", "parameter i out of range 1..ENGINE_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutEngMoneyCost(i, n);
    return LPtr();
}

// (put-engine-cost-t i:int<1..ENGINE_NR> n:int<0..10000>)
static LPtr
FPutEngineCostT (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > ENGINE_NR)
        throw LLispException("range-error", "parameter i out of range 1..ENGINE_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutEngTritCost(i, n);
    return LPtr();
}

// (put-engine-cost-d i:int<1..ENGINE_NR> n:int<0..10000>)
static LPtr
FPutEngineCostD (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > ENGINE_NR)
        throw LLispException("range-error", "parameter i out of range 1..ENGINE_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutEngDurCost(i, n);
    return LPtr();
}

// (put-engine-cost-m i:int<1..ENGINE_NR> n:int<0..10000>)
static LPtr
FPutEngineCostM (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > ENGINE_NR)
        throw LLispException("range-error", "parameter i out of range 1..ENGINE_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutEngMolyCost(i, n);
    return LPtr();
}

// (put-engine-tech i:int<1..ENGINE_NR> n:int<1..10>)
static LPtr
FPutEngineTech (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > ENGINE_NR)
        throw LLispException("range-error", "parameter i out of range 1..ENGINE_NR");
    if (n < 1 || n > 10)
        throw LLispException("range-error", "parameter n out of range 1..10");
    PutEngTechLevel(i, n);
    return LPtr();
}

// (put-engine-fuel-usage i:int<1..ENGINE_NR> sp:int<0..MAX_SPEED> val:int<0..1000000>)
static LPtr
FPutEngineFuelUsage (LPtr p)
{
    int i = argInt(p);
    int sp = argInt(p);
    int val = argInt(p);
    if (i < 1 || i > ENGINE_NR)
        throw LLispException("range-error", "parameter i out of range 1..ENGINE_NR");
    if (sp < 0 || sp > MAX_SPEED)
        throw LLispException("range-error", "parameter sp out of range 0..MAX_SPEED");
    if (val < 0 || val > 1000000)
        throw LLispException("range-error", "parameter val out of range 0..1000000");
    PutEngFuelConsumption(i, sp, val);
    return LPtr();
}

// (torp-name i:int<1..TORP_NR>)
static LPtr
FTorpName (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > TORP_NR)
        throw LLispException("range-error", "parameter i out of range 1..TORP_NR");
    return new LString(TorpName(i, 0));
}

// (torp-cost i:int<1..TORP_NR>)
static LPtr
FTorpCost (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > TORP_NR)
        throw LLispException("range-error", "parameter i out of range 1..TORP_NR");
    return new LInt(TorpTorpCost(i));
}

// (torp-tube-cost-mc i:int<1..TORP_NR>)
static LPtr
FTorpTubeCostMc (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > TORP_NR)
        throw LLispException("range-error", "parameter i out of range 1..TORP_NR");
    return new LInt(TorpTubeCost(i));
}

// (torp-tube-cost-t i:int<1..TORP_NR>)
static LPtr
FTorpTubeCostT (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > TORP_NR)
        throw LLispException("range-error", "parameter i out of range 1..TORP_NR");
    return new LInt(TorpTritCost(i));
}

// (torp-tube-cost-d i:int<1..TORP_NR>)
static LPtr
FTorpTubeCostD (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > TORP_NR)
        throw LLispException("range-error", "parameter i out of range 1..TORP_NR");
    return new LInt(TorpDurCost(i));
}

// (torp-tube-cost-m i:int<1..TORP_NR>)
static LPtr
FTorpTubeCostM (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > TORP_NR)
        throw LLispException("range-error", "parameter i out of range 1..TORP_NR");
    return new LInt(TorpMolyCost(i));
}

// (torp-tube-mass i:int<1..TORP_NR>)
static LPtr
FTorpTubeMass (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > TORP_NR)
        throw LLispException("range-error", "parameter i out of range 1..TORP_NR");
    return new LInt(TorpTubeMass(i));
}

// (torp-tech i:int<1..TORP_NR>)
static LPtr
FTorpTech (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > TORP_NR)
        throw LLispException("range-error", "parameter i out of range 1..TORP_NR");
    return new LInt(TorpTechLevel(i));
}

// (torp-kill i:int<1..TORP_NR>)
static LPtr
FTorpKill (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > TORP_NR)
        throw LLispException("range-error", "parameter i out of range 1..TORP_NR");
    return new LInt(TorpKillPower(i));
}

// (torp-expl i:int<1..TORP_NR>)
static LPtr
FTorpExpl (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > TORP_NR)
        throw LLispException("range-error", "parameter i out of range 1..TORP_NR");
    return new LInt(TorpDestructivePower(i));
}

// (put-torp-name i:int<1..TORP_NR> n:string)
static LPtr
FPutTorpName (LPtr p)
{
    int i = argInt(p);
    std::string n = argString(p);
    if (i < 1 || i > TORP_NR)
        throw LLispException("range-error", "parameter i out of range 1..TORP_NR");
    PutTorpName(i, n.c_str());
    return LPtr();
}

// (put-torp-cost i:int<1..TORP_NR> n:int<0..10000>)
static LPtr
FPutTorpCost (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > TORP_NR)
        throw LLispException("range-error", "parameter i out of range 1..TORP_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutTorpTorpCost(i, n);
    return LPtr();
}

// (put-torp-tube-cost-mc i:int<1..TORP_NR> n:int<0..10000>)
static LPtr
FPutTorpTubeCostMc (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > TORP_NR)
        throw LLispException("range-error", "parameter i out of range 1..TORP_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutTorpTubeCost(i, n);
    return LPtr();
}

// (put-torp-tube-cost-t i:int<1..TORP_NR> n:int<0..10000>)
static LPtr
FPutTorpTubeCostT (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > TORP_NR)
        throw LLispException("range-error", "parameter i out of range 1..TORP_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutTorpTritCost(i, n);
    return LPtr();
}

// (put-torp-tube-cost-d i:int<1..TORP_NR> n:int<0..10000>)
static LPtr
FPutTorpTubeCostD (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > TORP_NR)
        throw LLispException("range-error", "parameter i out of range 1..TORP_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutTorpDurCost(i, n);
    return LPtr();
}

// (put-torp-tube-cost-m i:int<1..TORP_NR> n:int<0..10000>)
static LPtr
FPutTorpTubeCostM (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > TORP_NR)
        throw LLispException("range-error", "parameter i out of range 1..TORP_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutTorpMolyCost(i, n);
    return LPtr();
}

// (put-torp-tube-mass i:int<1..TORP_NR> n:int<0..10000>)
static LPtr
FPutTorpTubeMass (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > TORP_NR)
        throw LLispException("range-error", "parameter i out of range 1..TORP_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutTorpTubeMass(i, n);
    return LPtr();
}

// (put-torp-tech i:int<1..TORP_NR> n:int<0..10000>)
static LPtr
FPutTorpTech (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > TORP_NR)
        throw LLispException("range-error", "parameter i out of range 1..TORP_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutTorpTechLevel(i, n);
    return LPtr();
}

// (put-torp-kill i:int<1..TORP_NR> n:int<0..10000>)
static LPtr
FPutTorpKill (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > TORP_NR)
        throw LLispException("range-error", "parameter i out of range 1..TORP_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutTorpKillPower(i, n);
    return LPtr();
}

// (put-torp-expl i:int<1..TORP_NR> n:int<0..10000>)
static LPtr
FPutTorpExpl (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > TORP_NR)
        throw LLispException("range-error", "parameter i out of range 1..TORP_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutTorpDestructivePower(i, n);
    return LPtr();
}

// (beam-name i:int<1..BEAM_NR>)
static LPtr
FBeamName (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > BEAM_NR)
        throw LLispException("range-error", "parameter i out of range 1..BEAM_NR");
    return new LString(BeamName(i,0));
}

// (beam-cost-mc i:int<1..BEAM_NR>)
static LPtr
FBeamCostMc (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > BEAM_NR)
        throw LLispException("range-error", "parameter i out of range 1..BEAM_NR");
    return new LInt(BeamMoneyCost(i));
}

// (beam-cost-t i:int<1..BEAM_NR>)
static LPtr
FBeamCostT (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > BEAM_NR)
        throw LLispException("range-error", "parameter i out of range 1..BEAM_NR");
    return new LInt(BeamTritCost(i));
}

// (beam-cost-d i:int<1..BEAM_NR>)
static LPtr
FBeamCostD (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > BEAM_NR)
        throw LLispException("range-error", "parameter i out of range 1..BEAM_NR");
    return new LInt(BeamDurCost(i));
}

// (beam-cost-m i:int<1..BEAM_NR>)
static LPtr
FBeamCostM (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > BEAM_NR)
        throw LLispException("range-error", "parameter i out of range 1..BEAM_NR");
    return new LInt(BeamMolyCost(i));
}

// (beam-mass i:int<1..BEAM_NR>)
static LPtr
FBeamMass (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > BEAM_NR)
        throw LLispException("range-error", "parameter i out of range 1..BEAM_NR");
    return new LInt(BeamMass(i));
}

// (beam-tech i:int<1..BEAM_NR>)
static LPtr
FBeamTech (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > BEAM_NR)
        throw LLispException("range-error", "parameter i out of range 1..BEAM_NR");
    return new LInt(BeamTechLevel(i));
}

// (beam-kill i:int<1..BEAM_NR>)
static LPtr
FBeamKill (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > BEAM_NR)
        throw LLispException("range-error", "parameter i out of range 1..BEAM_NR");
    return new LInt(BeamKillPower(i));
}

// (beam-expl i:int<1..BEAM_NR>)
static LPtr
FBeamExpl (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > BEAM_NR)
        throw LLispException("range-error", "parameter i out of range 1..BEAM_NR");
    return new LInt(BeamDestructivePower(i));
}

// (put-beam-name i:int<1..BEAM_NR> n:string)
static LPtr
FPutBeamName (LPtr p)
{
    int i = argInt(p);
    std::string n = argString(p);
    if (i < 1 || i > BEAM_NR)
        throw LLispException("range-error", "parameter i out of range 1..BEAM_NR");
    PutBeamName(i, n.c_str());
    return LPtr();
}

// (put-beam-cost-mc i:int<1..BEAM_NR> n:int<0..10000>)
static LPtr
FPutBeamCostMc (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > BEAM_NR)
        throw LLispException("range-error", "parameter i out of range 1..BEAM_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutBeamMoneyCost(i, n);
    return LPtr();
}

// (put-beam-cost-t i:int<1..BEAM_NR> n:int<0..10000>)
static LPtr
FPutBeamCostT (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > BEAM_NR)
        throw LLispException("range-error", "parameter i out of range 1..BEAM_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutBeamTritCost(i, n);
    return LPtr();
}

// (put-beam-cost-d i:int<1..BEAM_NR> n:int<0..10000>)
static LPtr
FPutBeamCostD (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > BEAM_NR)
        throw LLispException("range-error", "parameter i out of range 1..BEAM_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutBeamDurCost(i, n);
    return LPtr();
}

// (put-beam-cost-m i:int<1..BEAM_NR> n:int<0..10000>)
static LPtr
FPutBeamCostM (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > BEAM_NR)
        throw LLispException("range-error", "parameter i out of range 1..BEAM_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutBeamMolyCost(i, n);
    return LPtr();
}

// (put-beam-mass i:int<1..BEAM_NR> n:int<0..10000>)
static LPtr
FPutBeamMass (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > BEAM_NR)
        throw LLispException("range-error", "parameter i out of range 1..BEAM_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutBeamMass(i, n);
    return LPtr();
}

// (put-beam-tech i:int<1..BEAM_NR> n:int<0..10000>)
static LPtr
FPutBeamTech (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > BEAM_NR)
        throw LLispException("range-error", "parameter i out of range 1..BEAM_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutBeamTechLevel(i, n);
    return LPtr();
}

// (put-beam-kill i:int<1..BEAM_NR> n:int<0..10000>)
static LPtr
FPutBeamKill (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > BEAM_NR)
        throw LLispException("range-error", "parameter i out of range 1..BEAM_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutBeamKillPower(i, n);
    return LPtr();
}

// (put-beam-expl i:int<1..BEAM_NR> n:int<0..10000>)
static LPtr
FPutBeamExpl (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > BEAM_NR)
        throw LLispException("range-error", "parameter i out of range 1..BEAM_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutBeamDestructivePower(i, n);
    return LPtr();
}

// (hull-name i:int<1..HULL_NR>)
static LPtr
FHullName (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    return new LString(HullName(i, 0));
}

// (hull-picture i:int<1..HULL_NR>)
static LPtr
FHullPicture (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    return new LInt(HullPicnumber(i));
}

// (hull-cost-t i:int<1..HULL_NR>)
static LPtr
FHullCostT (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    return new LInt(HullTritCost(i));
}

// (hull-cost-d i:int<1..HULL_NR>)
static LPtr
FHullCostD (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    return new LInt(HullDurCost(i));
}

// (hull-cost-m i:int<1..HULL_NR>)
static LPtr
FHullCostM (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    return new LInt(HullMolyCost(i));
}

// (hull-max-fuel i:int<1..HULL_NR>)
static LPtr
FHullMaxFuel (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    return new LInt(HullFuelCapacity(i));
}

// (hull-max-crew i:int<1..HULL_NR>)
static LPtr
FHullMaxCrew (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    return new LInt(HullCrewComplement(i));
}

// (hull-engine-number i:int<1..HULL_NR>)
static LPtr
FHullEngineNumber (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    return new LInt(HullEngineNumber(i));
}

// (hull-mass i:int<1..HULL_NR>)
static LPtr
FHullMass (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    return new LInt(HullMass(i));
}

// (hull-tech i:int<1..HULL_NR>)
static LPtr
FHullTech (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    return new LInt(HullTechLevel(i));
}

// (hull-max-cargo i:int<1..HULL_NR>)
static LPtr
FHullMaxCargo (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    return new LInt(HullCargoCapacity(i));
}

// (hull-bays i:int<1..HULL_NR>)
static LPtr
FHullBays (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    return new LInt(HullBays(i));
}

// (hull-max-tubes i:int<1..HULL_NR>)
static LPtr
FHullMaxTubes (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    return new LInt(HullTubeNumber(i));
}

// (hull-max-beams i:int<1..HULL_NR>)
static LPtr
FHullMaxBeams (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    return new LInt(HullBeamNumber(i));
}

// (hull-cost-mc i:int<1..HULL_NR>)
static LPtr
FHullCostMc (LPtr p)
{
    int i = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    return new LInt(HullMolyCost(i));
}

// (put-hull-name i:int<1..HULL_NR> n:string)
static LPtr
FPutHullName (LPtr p)
{
    int i = argInt(p);
    std::string n = argString(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    PutHullName(i, n.c_str());
    return LPtr();
}

// (put-hull-picture i:int<1..HULL_NR> p:int<0..10000>)
static LPtr
FPutHullPicture (LPtr p1)
{
    int i = argInt(p1);
    int p = argInt(p1);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    if (p < 0 || p > 10000)
        throw LLispException("range-error", "parameter p out of range 0..10000");
    PutHullPicnumber(i, p);
    return LPtr();
}

// (put-hull-cost-t i:int<1..HULL_NR> n:int<0..10000>)
static LPtr
FPutHullCostT (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutHullTritCost(i, n);
    return LPtr();
}

// (put-hull-cost-d i:int<1..HULL_NR> n:int<0..10000>)
static LPtr
FPutHullCostD (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutHullDurCost(i, n);
    return LPtr();
}

// (put-hull-cost-m i:int<1..HULL_NR> n:int<0..10000>)
static LPtr
FPutHullCostM (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutHullMolyCost(i, n);
    return LPtr();
}

// (put-hull-max-fuel i:int<1..HULL_NR> n:int<0..10000>)
static LPtr
FPutHullMaxFuel (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutHullFuelCapacity(i, n);
    return LPtr();
}

// (put-hull-max-crew i:int<1..HULL_NR> n:int<0..10000>)
static LPtr
FPutHullMaxCrew (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutHullCrewComplement(i, n);
    return LPtr();
}

// (put-hull-engine-number i:int<1..HULL_NR> n:int<0..1000>)
static LPtr
FPutHullEngineNumber (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    if (n < 0 || n > 1000)
        throw LLispException("range-error", "parameter n out of range 0..1000");
    PutHullEngineNumber(i, n);
    return LPtr();
}

// (put-hull-mass i:int<1..HULL_NR> n:int<0..10000>)
static LPtr
FPutHullMass (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutHullMass(i, n);
    return LPtr();
}

// (put-hull-tech i:int<1..HULL_NR> n:int<1..10>)
static LPtr
FPutHullTech (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    if (n < 1 || n > 10)
        throw LLispException("range-error", "parameter n out of range 1..10");
    PutHullTechLevel(i, n);
    return LPtr();
}

// (put-hull-max-cargo i:int<1..HULL_NR> n:int<0..10000>)
static LPtr
FPutHullMaxCargo (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutHullCargoCapacity(i, n);
    return LPtr();
}

// (put-hull-bays i:int<1..HULL_NR> n:int<0..10000>)
static LPtr
FPutHullBays (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutHullBays(i, n);
    return LPtr();
}

// (put-hull-max-tubes i:int<1..HULL_NR> n:int<0..10000>)
static LPtr
FPutHullMaxTubes (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutHullTubeNumber(i, n);
    return LPtr();
}

// (put-hull-max-beams i:int<1..HULL_NR> n:int<0..10000>)
static LPtr
FPutHullMaxBeams (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutHullBeamNumber(i, n);
    return LPtr();
}

// (put-hull-cost-mc i:int<1..HULL_NR> n:int<0..10000>)
static LPtr
FPutHullCostMc (LPtr p)
{
    int i = argInt(p);
    int n = argInt(p);
    if (i < 1 || i > HULL_NR)
        throw LLispException("range-error", "parameter i out of range 1..HULL_NR");
    if (n < 0 || n > 10000)
        throw LLispException("range-error", "parameter n out of range 0..10000");
    PutHullMoneyCost(i, n);
    return LPtr();
}

// (player-hull pl:player slot:int<1..RACEHULLS>)
static LPtr
FPlayerHull (LPtr p)
{
    RaceType_Def pl = argPlayer(p);
    int slot = argInt(p);
    if (slot < 1 || slot > RACEHULLS)
        throw LLispException("range-error", "parameter slot out of range 1..RACEHULLS");
    return new LInt(TrueHull(pl, slot));
}

// (put-player-hull pl:player slot:int<1..RACEHULLS> val:int<0..HULL_NR>)
static LPtr
FPutPlayerHull (LPtr p)
{
    RaceType_Def pl = argPlayer(p);
    int slot = argInt(p);
    int val = argInt(p);
    if (slot < 1 || slot > RACEHULLS)
        throw LLispException("range-error", "parameter slot out of range 1..RACEHULLS");
    if (val < 0 || val > HULL_NR)
        throw LLispException("range-error", "parameter val out of range 0..HULL_NR");
    PutTrueHull(pl, slot, val);
    return LPtr();
}

// (write-external-message pl:player message:string)
static LPtr
FWriteExternalMessage (LPtr p)
{
    RaceType_Def pl = argPlayer(p);
    std::string message = argString(p);
    return retBool(WriteExternalMessage(pl, message.c_str()));
}

// (write-auxhost-message pl:player message:string)
static LPtr
FWriteAuxhostMessage (LPtr p)
{
    RaceType_Def pl = argPlayer(p);
    std::string message = argString(p);
    return retBool(WriteAUXHOSTMessage(pl, message.c_str()));
}

// (random-init seed:int)
static LPtr
FRandomInit (LPtr p)
{
    int seed = argInt(p);
    SetRandomSeed(seed);
    return LPtr();
}

// (random-range max:int)
static LPtr
FRandomRange (LPtr p)
{
    int max = argInt(p);
    return new LInt(RandomRange(max));
}

// (ship-rc-start s:sid p:player)
static LPtr
FShipRcStart (LPtr p1)
{
    int s = argSid(p1);
    RaceType_Def p = argPlayer(p1);
    return retBool(BeginShipRemoteControl(s, p));
}

// (ship-rc-end s:sid p:player)
static LPtr
FShipRcEnd (LPtr p1)
{
    int s = argSid(p1);
    RaceType_Def p = argPlayer(p1);
    return retBool(EndShipRemoteControl(s, p));
}

// (ship-rc-p s:sid)
static LPtr
FShipRcP (LPtr p)
{
    int s = argSid(p);
    return retBool(IsShipRemoteControlled(s));
}

// (ship-rc-allowed-p s:sid)
static LPtr
FShipRcAllowedP (LPtr p)
{
    int s = argSid(p);
    return retBool(!IsShipRemoteForbidden(s));
}

// (ship-rc-reset s:sid)
static LPtr
FShipRcReset (LPtr p)
{
    int s = argSid(p);
    ResetShipRemoteControl(s);
    return LPtr();
}

// (ship-rc-set-default s:sid)
static LPtr
FShipRcSetDefault (LPtr p)
{
    int s = argSid(p);
    AssignDefaultForbidState(s);
    return LPtr();
}

// (put-ship-rc-allowed-p s:sid val:bool)
static LPtr
FPutShipRcAllowedP (LPtr p)
{
    int s = argSid(p);
    bool val = argBool(p);
    if (val)
        AllowShipRemoteControl(s, ShipOwner(s));
    else
        ForbidShipRemoteControl(s, ShipOwner(s));
    return LPtr();
}

// (ship-rc-controller s:sid)
static LPtr
FShipRcController (LPtr p)
{
    int s = argSid(p);
    return new LInt(ShipRemoteController(s));
}

// (ship-rc-owner s:sid)
static LPtr
FShipRcOwner (LPtr p)
{
    int s = argSid(p);
    return new LInt(ShipRemoteOwner(s));
}

// (player-rc-default p:player)
static LPtr
FPlayerRcDefault (LPtr p1)
{
    RaceType_Def p = argPlayer(p1);
    return retBool(!DefaultRemoteState(p));
}

// (put-player-rc-default p:player val:bool)
static LPtr
FPutPlayerRcDefault (LPtr p1)
{
    RaceType_Def p = argPlayer(p1);
    bool val = argBool(p1);
    SetDefaultRemoteState(p, Boolean(!val));
    return LPtr();
}

// (worm-count )
static LPtr
FWormCount (LPtr )
{
    return new LInt(NumWormholes());
}

// (make-worm )
static LPtr
FMakeWorm (LPtr )
{
    return intOrNull(CreateWormhole());
}

// (worm-delete i:wid)
static LPtr
FWormDelete (LPtr p)
{
    int i = argWid(p);
    DeleteWormhole(i);
    return LPtr();
}

// (worm-stability i:wid)
static LPtr
FWormStability (LPtr p)
{
    int i = argWid(p);
    return new LInt(WormholeStabilityCode(i));
}

// (worm-start-x i:wid)
static LPtr
FWormStartX (LPtr p)
{
    int i = argWid(p);
    return new LInt(WormholeStartX(i));
}

// (worm-start-y i:wid)
static LPtr
FWormStartY (LPtr p)
{
    int i = argWid(p);
    return new LInt(WormholeStartY(i));
}

// (worm-end-x i:wid)
static LPtr
FWormEndX (LPtr p)
{
    int i = argWid(p);
    return new LInt(WormholeEndX(i));
}

// (worm-end-y i:wid)
static LPtr
FWormEndY (LPtr p)
{
    int i = argWid(p);
    return new LInt(WormholeEndY(i));
}

// (worm-mass i:wid)
static LPtr
FWormMass (LPtr p)
{
    int i = argWid(p);
    return new LInt(WormholeMass(i));
}

// (worm-instability i:wid)
static LPtr
FWormInstability (LPtr p)
{
    int i = argWid(p);
    return new LInt(int(WormholeInstability(i)));
}

// (worm-start-waypoint-x i:wid)
static LPtr
FWormStartWaypointX (LPtr p)
{
    int i = argWid(p);
    return new LInt(WormholeWaypointStartX(i));
}

// (worm-start-waypoint-y i:wid)
static LPtr
FWormStartWaypointY (LPtr p)
{
    int i = argWid(p);
    return new LInt(WormholeWaypointStartY(i));
}

// (worm-end-waypoint-x i:wid)
static LPtr
FWormEndWaypointX (LPtr p)
{
    int i = argWid(p);
    return new LInt(WormholeWaypointEndX(i));
}

// (worm-end-waypoint-y i:wid)
static LPtr
FWormEndWaypointY (LPtr p)
{
    int i = argWid(p);
    return new LInt(WormholeWaypointEndY(i));
}

// (put-worm-start-x i:wid x:int<0..MAX_COORDINATE>)
static LPtr
FPutWormStartX (LPtr p)
{
    int i = argWid(p);
    int x = argInt(p);
    if (x < 0 || x > MAX_COORDINATE)
        throw LLispException("range-error", "parameter x out of range 0..MAX_COORDINATE");
    PutWormholeStartX(i, x);
    return LPtr();
}

// (put-worm-start-y i:wid x:int<0..MAX_COORDINATE>)
static LPtr
FPutWormStartY (LPtr p)
{
    int i = argWid(p);
    int x = argInt(p);
    if (x < 0 || x > MAX_COORDINATE)
        throw LLispException("range-error", "parameter x out of range 0..MAX_COORDINATE");
    PutWormholeStartY(i, x);
    return LPtr();
}

// (put-worm-end-x i:wid x:int<0..MAX_COORDINATE>)
static LPtr
FPutWormEndX (LPtr p)
{
    int i = argWid(p);
    int x = argInt(p);
    if (x < 0 || x > MAX_COORDINATE)
        throw LLispException("range-error", "parameter x out of range 0..MAX_COORDINATE");
    PutWormholeEndX(i, x);
    return LPtr();
}

// (put-worm-end-y i:wid x:int<0..MAX_COORDINATE>)
static LPtr
FPutWormEndY (LPtr p)
{
    int i = argWid(p);
    int x = argInt(p);
    if (x < 0 || x > MAX_COORDINATE)
        throw LLispException("range-error", "parameter x out of range 0..MAX_COORDINATE");
    PutWormholeEndY(i, x);
    return LPtr();
}

// (put-worm-mass i:wid x:int<-32767..32767>)
static LPtr
FPutWormMass (LPtr p)
{
    int i = argWid(p);
    int x = argInt(p);
    if (x < -32767 || x > 32767)
        throw LLispException("range-error", "parameter x out of range -32767..32767");
    if (x == 0)
       throw LLispException("range-error", "Wormhole mass cannot be 0");
    PutWormholeMass(i, x);
    return LPtr();
}

// (put-worm-instability i:wid x:double<0..100>)
static LPtr
FPutWormInstability (LPtr p)
{
    int i = argWid(p);
    int x = argInt(p);
    if (x < 0 || x > 100)
        throw LLispException("range-error", "parameter x out of range 0..100");
    PutWormholeInstability(i, x);
    return LPtr();
}

// (put-worm-start-waypoint-x i:wid x:int<0..MAX_COORDINATE>)
static LPtr
FPutWormStartWaypointX (LPtr p)
{
    int i = argWid(p);
    int x = argInt(p);
    if (x < 0 || x > MAX_COORDINATE)
        throw LLispException("range-error", "parameter x out of range 0..MAX_COORDINATE");
    PutWormholeWaypointStartX(i, x);
    return LPtr();
}

// (put-worm-start-waypoint-y i:wid x:int<0..MAX_COORDINATE>)
static LPtr
FPutWormStartWaypointY (LPtr p)
{
    int i = argWid(p);
    int x = argInt(p);
    if (x < 0 || x > MAX_COORDINATE)
        throw LLispException("range-error", "parameter x out of range 0..MAX_COORDINATE");
    PutWormholeWaypointStartY(i, x);
    return LPtr();
}

// (put-worm-end-waypoint-x i:wid x:int<0..MAX_COORDINATE>)
static LPtr
FPutWormEndWaypointX (LPtr p)
{
    int i = argWid(p);
    int x = argInt(p);
    if (x < 0 || x > MAX_COORDINATE)
        throw LLispException("range-error", "parameter x out of range 0..MAX_COORDINATE");
    PutWormholeWaypointEndX(i, x);
    return LPtr();
}

// (put-worm-end-waypoint-y i:wid x:int<0..MAX_COORDINATE>)
static LPtr
FPutWormEndWaypointY (LPtr p)
{
    int i = argWid(p);
    int x = argInt(p);
    if (x < 0 || x > MAX_COORDINATE)
        throw LLispException("range-error", "parameter x out of range 0..MAX_COORDINATE");
    PutWormholeWaypointEndY(i, x);
    return LPtr();
}

// (ship-exp-level s:sexp)
static LPtr
FShipExpLevel (LPtr p)
{
    int s = argSid(p);
    return new LInt(ExperienceLevel(ShipExperience(s)));
}

// (ship-exp-add s:sexp add:int)
static LPtr
FShipExpAdd (LPtr p)
{
    int s = argSid(p);
    int add = argInt(p);
    AddToExperience(ShipExperience(s), add);
    return LPtr();
}

// (ship-exp-points s:sexp)
static LPtr
FShipExpPoints (LPtr p)
{
    int s = argSid(p);
    return new LInt(ExperiencePoints(ShipExperience(s)));
}

// (put-ship-exp-points s:sexp n:int)
static LPtr
FPutShipExpPoints (LPtr p)
{
    int s = argSid(p);
    int n = argInt(p);
    PutExperiencePoints(ShipExperience(s), n);
    return LPtr();
}

// (ship-exp-new-points s:sexp)
static LPtr
FShipExpNewPoints (LPtr p)
{
    int s = argSid(p);
    return new LInt(NewExperiencePoints(ShipExperience(s)));
}

// (planet-exp-level s:pexp)
static LPtr
FPlanetExpLevel (LPtr p)
{
    int s = argPid(p);
    return new LInt(ExperienceLevel(PlanetExperience(s)));
}

// (planet-exp-add s:pexp add:int)
static LPtr
FPlanetExpAdd (LPtr p)
{
    int s = argPid(p);
    int add = argInt(p);
    AddToExperience(PlanetExperience(s), add);
    return LPtr();
}

// (planet-exp-points s:pexp)
static LPtr
FPlanetExpPoints (LPtr p)
{
    int s = argPid(p);
    return new LInt(ExperiencePoints(PlanetExperience(s)));
}

// (put-planet-exp-points s:pexp n:int)
static LPtr
FPutPlanetExpPoints (LPtr p)
{
    int s = argPid(p);
    int n = argInt(p);
    PutExperiencePoints(PlanetExperience(s), n);
    return LPtr();
}

// (planet-exp-new-points s:pexp)
static LPtr
FPlanetExpNewPoints (LPtr p)
{
    int s = argPid(p);
    return new LInt(NewExperiencePoints(PlanetExperience(s)));
}

// (load-game )
static LPtr
FLoadGame (LPtr )
{
    if (!ReadHostData())
        throw LLispException("io-error", "Unable to read game data");
    return LPtr();
}

// (load-spec )
static LPtr
FLoadSpec (LPtr )
{
    if (!ReadGlobalData())
        throw LLispException("io-error", "Unable to read global data");
    return LPtr();
}

// (save-game )
static LPtr
FSaveGame (LPtr )
{
    if (!WriteHostData())
        throw LLispException("io-error", "Unable to write game data");
    return LPtr();
}

// (save-planets )
static LPtr
FSavePlanets (LPtr )
{
    return retBool(Write_Planets_File(PLANET_NR));
}

// (save-ships )
static LPtr
FSaveShips (LPtr )
{
    return retBool(Write_Ships_File(SHIP_NR));
}

// (save-bases )
static LPtr
FSaveBases (LPtr )
{
    return retBool(Write_Bases_File(SHIP_NR, TurnNumber()));
}

// (save-mines )
static LPtr
FSaveMines (LPtr )
{
    return retBool(Write_Mines_File());
}

// (save-gen )
static LPtr
FSaveGen (LPtr )
{
    return retBool(Write_HostGen_File());
}

// (save-aux )
static LPtr
FSaveAux (LPtr )
{
    return retBool(Write_AuxData_File());
}

// (save-racenames )
static LPtr
FSaveRacenames (LPtr )
{
    return retBool(Write_Racenames_File());
}

// (save-wormholes )
static LPtr
FSaveWormholes (LPtr )
{
    return retBool(WriteWormholeFile());
}

// (save-cloak-status )
static LPtr
FSaveCloakStatus (LPtr )
{
    return retBool(WriteCLOAKCFile());
}

// (save-hullspec )
static LPtr
FSaveHullspec (LPtr )
{
    return retBool(Write_Hullspec_File());
}

// (save-truehull )
static LPtr
FSaveTruehull (LPtr )
{
    return retBool(Write_Truehull_File());
}

// (save-engspec )
static LPtr
FSaveEngspec (LPtr )
{
    return retBool(Write_Engspec_File());
}

// (save-torpspec )
static LPtr
FSaveTorpspec (LPtr )
{
    return retBool(Write_Torpspec_File());
}

// (save-beamspec )
static LPtr
FSaveBeamspec (LPtr )
{
    return retBool(Write_Beamspec_File());
}

// (save-planet-names )
static LPtr
FSavePlanetNames (LPtr )
{
    return retBool(Write_Planetname_File());
}

// (save-planet-locations )
static LPtr
FSavePlanetLocations (LPtr )
{
    return retBool(Write_Xyplan_File());
}

// (ship-cloak-status-known-p )
static LPtr
FShipCloakStatusKnownP (LPtr )
{
    return retBool(IsCLOAKCFound());
}

// (init-planets )
static LPtr
FInitPlanets (LPtr )
{
    InitializePlanets();
    return LPtr();
}

// (init-ships )
static LPtr
FInitShips (LPtr )
{
    InitializeShips();
    return LPtr();
}

// (init-bases )
static LPtr
FInitBases (LPtr )
{
    InitializeBases();
    return LPtr();
}

// (init-mines )
static LPtr
FInitMines (LPtr )
{
    InitializeMines();
    return LPtr();
}

// (init-gen )
static LPtr
FInitGen (LPtr )
{
    InitializeHostGen();
    return LPtr();
}

// (init-version n : int)
static LPtr
FInitVersion (LPtr p)
{
    int n = argInt(p);
    SetNewMajorVersion(n);
    return LPtr();
}

// (util-set-mode mode:int<0..2>)
static LPtr
FUtilSetMode (LPtr p)
{
    int mode = argInt(p);
    if (mode < 0 || mode > 2)
        throw LLispException("range-error", "parameter mode out of range 0..2");
    SetUtilMode(UtilMode_Def(mode));
    return LPtr();
}

// (game-file-open-read name:string mode:int)
static LPtr
FGameFileOpenRead (LPtr p)
{
    std::string name = argString(p);
    int mode = argInt(p);
    return new LFile(OpenInputFile(name.c_str(), mode), true);
}

// (game-file-open-write name:string mode:int)
static LPtr
FGameFileOpenWrite (LPtr p)
{
    std::string name = argString(p);
    int mode = argInt(p);
    return new LFile(OpenOutputFile(name.c_str(), mode), false);
}

// (game-file-remove name:string)
static LPtr
FGameFileRemove (LPtr p)
{
    std::string name = argString(p);
    return retBool(RemoveGameFile(name.c_str()));
}

// (game-file-copy in:string out:string)
static LPtr
FGameFileCopy (LPtr p)
{
    std::string in = argString(p);
    std::string out = argString(p);
    return retBool(CopyGameFile(in.c_str(), out.c_str()));
}

// (game-file-rename fm:string to:string)
static LPtr
FGameFileRename (LPtr p)
{
    std::string fm = argString(p);
    std::string to = argString(p);
    return retBool(RenameGameFile(fm.c_str(), to.c_str()));
}

// (file-size f:file)
static LPtr
FFileSize (LPtr p)
{
    LFile* f = argFile(p);
    return new LInt(FileLength(f->getFile()));
}

static void
initGeneratedFunctions()
{
    LSymbol::define("ships-within-radius", new LBuiltin(FShipsWithinRadius, 3, 3));
    LSymbol::define("ships-at", new LBuiltin(FShipsAt, 2, 2));
    LSymbol::define("mines-within-radius", new LBuiltin(FMinesWithinRadius, 3, 3));
    LSymbol::define("planet-at-ship", new LBuiltin(FPlanetAtShip, 1, 1));
    LSymbol::define("gravity-planet", new LBuiltin(FGravityPlanet, 2, 2));
    LSymbol::define("planets-within-radius", new LBuiltin(FPlanetsWithinRadius, 3, 3));
    LSymbol::define("ships-at-planet", new LBuiltin(FShipsAtPlanet, 1, 1));
    LSymbol::define("mines-covering", new LBuiltin(FMinesCovering, 2, 2));
    LSymbol::define("mines-overlapping", new LBuiltin(FMinesOverlapping, 1, 1));
    LSymbol::define("player-active-p", new LBuiltin(FPlayerActiveP, 1, 1));
    LSymbol::define("put-player-active-p", new LBuiltin(FPutPlayerActiveP, 2, 2));
    LSymbol::define("player-password", new LBuiltin(FPlayerPassword, 1, 1));
    LSymbol::define("put-player-password", new LBuiltin(FPutPlayerPassword, 2, 2));
    LSymbol::define("kill-config", new LBuiltin(FKillConfig, 1, 1));
    LSymbol::define("put-kill-config", new LBuiltin(FPutKillConfig, 2, 2));
    LSymbol::define("kill-player", new LBuiltin(FKillPlayer, 1, 1));
    LSymbol::define("kill-player-planets", new LBuiltin(FKillPlayerPlanets, 1, 1));
    LSymbol::define("kill-player-bases", new LBuiltin(FKillPlayerBases, 1, 1));
    LSymbol::define("kill-player-minefields", new LBuiltin(FKillPlayerMinefields, 1, 1));
    LSymbol::define("kill-player-ships", new LBuiltin(FKillPlayerShips, 1, 1));
    LSymbol::define("eff-race", new LBuiltin(FEffRace, 1, 1));
    LSymbol::define("eff-mission", new LBuiltin(FEffMission, 1, 1));
    LSymbol::define("build-queue-invalidate", new LBuiltin(FBuildQueueInvalidate, 1, 1));
    LSymbol::define("ships-in-build-queue", new LBuiltin(FShipsInBuildQueue, 0, 0));
    LSymbol::define("sort-build-queue", new LBuiltin(FSortBuildQueue, 0, 0));
    LSymbol::define("build-queue-base-id", new LBuiltin(FBuildQueueBaseId, 1, 1));
    LSymbol::define("build-queue-priority", new LBuiltin(FBuildQueuePriority, 1, 1));
    LSymbol::define("build-queue-cloning-p", new LBuiltin(FBuildQueueCloningP, 1, 1));
    LSymbol::define("build-queue-ship-owner", new LBuiltin(FBuildQueueShipOwner, 1, 1));
    LSymbol::define("build-queue-owner", new LBuiltin(FBuildQueueOwner, 1, 1));
    LSymbol::define("build-queue-hull-type", new LBuiltin(FBuildQueueHullType, 1, 1));
    LSymbol::define("build-queue-engine-type", new LBuiltin(FBuildQueueEngineType, 1, 1));
    LSymbol::define("build-queue-beam-type", new LBuiltin(FBuildQueueBeamType, 1, 1));
    LSymbol::define("build-queue-torp-type", new LBuiltin(FBuildQueueTorpType, 1, 1));
    LSymbol::define("build-queue-beam-number", new LBuiltin(FBuildQueueBeamNumber, 1, 1));
    LSymbol::define("build-queue-torp-number", new LBuiltin(FBuildQueueTorpNumber, 1, 1));
    LSymbol::define("put-build-queue-priority", new LBuiltin(FPutBuildQueuePriority, 2, 2));
    LSymbol::define("activity-level", new LBuiltin(FActivityLevel, 1, 1));
    LSymbol::define("put-activity-level", new LBuiltin(FPutActivityLevel, 2, 2));
    LSymbol::define("ally-add", new LBuiltin(FAllyAdd, 2, 2));
    LSymbol::define("ally-drop", new LBuiltin(FAllyDrop, 2, 2));
    LSymbol::define("ally-allows-p", new LBuiltin(FAllyAllowsP, 3, 3));
    LSymbol::define("put-ally-allows", new LBuiltin(FPutAllyAllows, 4, 4));
    LSymbol::define("ally-offers-p", new LBuiltin(FAllyOffersP, 2, 2));
    LSymbol::define("players-are-allied-p", new LBuiltin(FPlayersAreAlliedP, 2, 2));
    LSymbol::define("ally-offer-conditional-p", new LBuiltin(FAllyOfferConditionalP, 3, 3));
    LSymbol::define("ally-offer-any-p", new LBuiltin(FAllyOfferAnyP, 3, 3));
    LSymbol::define("ship-exists-p", new LBuiltin(FShipExistsP, 1, 1));
    LSymbol::define("ship-delete", new LBuiltin(FShipDelete, 1, 1));
    LSymbol::define("ship-owner", new LBuiltin(FShipOwner, 1, 1));
    LSymbol::define("ship-speed", new LBuiltin(FShipSpeed, 1, 1));
    LSymbol::define("ship-location-x", new LBuiltin(FShipLocationX, 1, 1));
    LSymbol::define("ship-location-y", new LBuiltin(FShipLocationY, 1, 1));
    LSymbol::define("ship-waypoint-x", new LBuiltin(FShipWaypointX, 1, 1));
    LSymbol::define("ship-waypoint-y", new LBuiltin(FShipWaypointY, 1, 1));
    LSymbol::define("ship-waypoint-dx", new LBuiltin(FShipWaypointDx, 1, 1));
    LSymbol::define("ship-waypoint-dy", new LBuiltin(FShipWaypointDy, 1, 1));
    LSymbol::define("ship-hull", new LBuiltin(FShipHull, 1, 1));
    LSymbol::define("ship-engine", new LBuiltin(FShipEngine, 1, 1));
    LSymbol::define("ship-beam-type", new LBuiltin(FShipBeamType, 1, 1));
    LSymbol::define("ship-beam-number", new LBuiltin(FShipBeamNumber, 1, 1));
    LSymbol::define("ship-torp-type", new LBuiltin(FShipTorpType, 1, 1));
    LSymbol::define("ship-tube-number", new LBuiltin(FShipTubeNumber, 1, 1));
    LSymbol::define("ship-bays", new LBuiltin(FShipBays, 1, 1));
    LSymbol::define("ship-ammo", new LBuiltin(FShipAmmo, 1, 1));
    LSymbol::define("ship-mission", new LBuiltin(FShipMission, 1, 1));
    LSymbol::define("ship-enemy", new LBuiltin(FShipEnemy, 1, 1));
    LSymbol::define("ship-tow-target", new LBuiltin(FShipTowTarget, 1, 1));
    LSymbol::define("ship-intercept-target", new LBuiltin(FShipInterceptTarget, 1, 1));
    LSymbol::define("ship-damage", new LBuiltin(FShipDamage, 1, 1));
    LSymbol::define("ship-crew", new LBuiltin(FShipCrew, 1, 1));
    LSymbol::define("ship-unload-cargo", new LBuiltin(FShipUnloadCargo, 2, 2));
    LSymbol::define("ship-unload-target", new LBuiltin(FShipUnloadTarget, 1, 1));
    LSymbol::define("ship-transfer-cargo", new LBuiltin(FShipTransferCargo, 2, 2));
    LSymbol::define("ship-transfer-target", new LBuiltin(FShipTransferTarget, 1, 1));
    LSymbol::define("ship-name", new LBuiltin(FShipName, 1, 1));
    LSymbol::define("ship-fcode", new LBuiltin(FShipFcode, 1, 1));
    LSymbol::define("ship-cargo", new LBuiltin(FShipCargo, 2, 2));
    LSymbol::define("make-ship", new LBuiltin(FMakeShip, 1, 1));
    LSymbol::define("ship-mass", new LBuiltin(FShipMass, 1, 1));
    LSymbol::define("ship-engine-load", new LBuiltin(FShipEngineLoad, 1, 1));
    LSymbol::define("ship-cargo-mass", new LBuiltin(FShipCargoMass, 1, 1));
    LSymbol::define("ship-mission-str", new LBuiltin(FShipMissionStr, 1, 1));
    LSymbol::define("put-ship-cargo", new LBuiltin(FPutShipCargo, 3, 3));
    LSymbol::define("put-ship-name", new LBuiltin(FPutShipName, 2, 2));
    LSymbol::define("put-ship-owner", new LBuiltin(FPutShipOwner, 2, 2));
    LSymbol::define("put-ship-fcode", new LBuiltin(FPutShipFcode, 2, 2));
    LSymbol::define("put-ship-speed", new LBuiltin(FPutShipSpeed, 2, 2));
    LSymbol::define("put-ship-waypoint-x", new LBuiltin(FPutShipWaypointX, 2, 2));
    LSymbol::define("put-ship-waypoint-y", new LBuiltin(FPutShipWaypointY, 2, 2));
    LSymbol::define("put-ship-waypoint-dx", new LBuiltin(FPutShipWaypointDx, 2, 2));
    LSymbol::define("put-ship-waypoint-dy", new LBuiltin(FPutShipWaypointDy, 2, 2));
    LSymbol::define("put-ship-location-x", new LBuiltin(FPutShipLocationX, 2, 2));
    LSymbol::define("put-ship-location-y", new LBuiltin(FPutShipLocationY, 2, 2));
    LSymbol::define("put-ship-hull", new LBuiltin(FPutShipHull, 2, 2));
    LSymbol::define("put-ship-engine", new LBuiltin(FPutShipEngine, 2, 2));
    LSymbol::define("put-ship-beam-type", new LBuiltin(FPutShipBeamType, 2, 2));
    LSymbol::define("put-ship-beam-number", new LBuiltin(FPutShipBeamNumber, 2, 2));
    LSymbol::define("put-ship-torp-type", new LBuiltin(FPutShipTorpType, 2, 2));
    LSymbol::define("put-ship-tube-number", new LBuiltin(FPutShipTubeNumber, 2, 2));
    LSymbol::define("put-ship-bays", new LBuiltin(FPutShipBays, 2, 2));
    LSymbol::define("put-ship-ammo", new LBuiltin(FPutShipAmmo, 2, 2));
    LSymbol::define("put-ship-mission", new LBuiltin(FPutShipMission, 2, 2));
    LSymbol::define("put-ship-enemy", new LBuiltin(FPutShipEnemy, 2, 2));
    LSymbol::define("put-ship-tow-target", new LBuiltin(FPutShipTowTarget, 2, 2));
    LSymbol::define("put-ship-intercept-target", new LBuiltin(FPutShipInterceptTarget, 2, 2));
    LSymbol::define("put-ship-damage", new LBuiltin(FPutShipDamage, 2, 2));
    LSymbol::define("put-ship-crew", new LBuiltin(FPutShipCrew, 2, 2));
    LSymbol::define("put-ship-unload-cargo", new LBuiltin(FPutShipUnloadCargo, 3, 3));
    LSymbol::define("put-ship-unload-target", new LBuiltin(FPutShipUnloadTarget, 2, 2));
    LSymbol::define("put-ship-transfer-cargo", new LBuiltin(FPutShipTransferCargo, 3, 3));
    LSymbol::define("put-ship-transfer-target", new LBuiltin(FPutShipTransferTarget, 2, 2));
    LSymbol::define("ship-cloaked-p", new LBuiltin(FShipCloakedP, 1, 1));
    LSymbol::define("put-ship-cloaked-p", new LBuiltin(FPutShipCloakedP, 2, 2));
    LSymbol::define("ship-shield-level", new LBuiltin(FShipShieldLevel, 1, 1));
    LSymbol::define("put-ship-shield-level", new LBuiltin(FPutShipShieldLevel, 2, 2));
    LSymbol::define("planet-exists-p", new LBuiltin(FPlanetExistsP, 1, 1));
    LSymbol::define("planet-location-x", new LBuiltin(FPlanetLocationX, 1, 1));
    LSymbol::define("planet-location-y", new LBuiltin(FPlanetLocationY, 1, 1));
    LSymbol::define("planet-owner", new LBuiltin(FPlanetOwner, 1, 1));
    LSymbol::define("planet-mines", new LBuiltin(FPlanetMines, 1, 1));
    LSymbol::define("planet-defense", new LBuiltin(FPlanetDefense, 1, 1));
    LSymbol::define("planet-factories", new LBuiltin(FPlanetFactories, 1, 1));
    LSymbol::define("planet-cargo", new LBuiltin(FPlanetCargo, 2, 2));
    LSymbol::define("planet-core", new LBuiltin(FPlanetCore, 2, 2));
    LSymbol::define("planet-density", new LBuiltin(FPlanetDensity, 2, 2));
    LSymbol::define("planet-colonist-tax", new LBuiltin(FPlanetColonistTax, 1, 1));
    LSymbol::define("planet-native-tax", new LBuiltin(FPlanetNativeTax, 1, 1));
    LSymbol::define("planet-colonist-happy", new LBuiltin(FPlanetColonistHappy, 1, 1));
    LSymbol::define("planet-native-happy", new LBuiltin(FPlanetNativeHappy, 1, 1));
    LSymbol::define("planet-native-gov", new LBuiltin(FPlanetNativeGov, 1, 1));
    LSymbol::define("planet-native-race", new LBuiltin(FPlanetNativeRace, 1, 1));
    LSymbol::define("planet-native-pop", new LBuiltin(FPlanetNativePop, 1, 1));
    LSymbol::define("planet-temp", new LBuiltin(FPlanetTemp, 1, 1));
    LSymbol::define("planet-build-base-p", new LBuiltin(FPlanetBuildBaseP, 1, 1));
    LSymbol::define("planet-has-natives-p", new LBuiltin(FPlanetHasNativesP, 1, 1));
    LSymbol::define("planet-native-race-str", new LBuiltin(FPlanetNativeRaceStr, 1, 1));
    LSymbol::define("planet-native-gov-str", new LBuiltin(FPlanetNativeGovStr, 1, 1));
    LSymbol::define("planet-factories-max", new LBuiltin(FPlanetFactoriesMax, 1, 1));
    LSymbol::define("planet-defense-max", new LBuiltin(FPlanetDefenseMax, 1, 1));
    LSymbol::define("planet-mines-max", new LBuiltin(FPlanetMinesMax, 1, 1));
    LSymbol::define("planet-mining", new LBuiltin(FPlanetMining, 2, 2));
    LSymbol::define("planet-native-gov-tax", new LBuiltin(FPlanetNativeGovTax, 1, 1));
    LSymbol::define("planet-native-happy-change", new LBuiltin(FPlanetNativeHappyChange, 1, 1));
    LSymbol::define("planet-colonist-happy-change", new LBuiltin(FPlanetColonistHappyChange, 1, 1));
    LSymbol::define("planet-native-income", new LBuiltin(FPlanetNativeIncome, 1, 1));
    LSymbol::define("planet-colonist-income", new LBuiltin(FPlanetColonistIncome, 1, 1));
    LSymbol::define("planet-supply-production", new LBuiltin(FPlanetSupplyProduction, 1, 1));
    LSymbol::define("planet-num-ships-hissing", new LBuiltin(FPlanetNumShipsHissing, 1, 1));
    LSymbol::define("planet-name", new LBuiltin(FPlanetName, 1, 1));
    LSymbol::define("planet-fcode", new LBuiltin(FPlanetFcode, 1, 1));
    LSymbol::define("planet-temp-str", new LBuiltin(FPlanetTempStr, 1, 1));
    LSymbol::define("put-planet-owner", new LBuiltin(FPutPlanetOwner, 2, 2));
    LSymbol::define("put-planet-owner-only", new LBuiltin(FPutPlanetOwnerOnly, 2, 2));
    LSymbol::define("put-planet-fcode", new LBuiltin(FPutPlanetFcode, 2, 2));
    LSymbol::define("put-planet-mines", new LBuiltin(FPutPlanetMines, 2, 2));
    LSymbol::define("put-planet-factories", new LBuiltin(FPutPlanetFactories, 2, 2));
    LSymbol::define("put-planet-defense", new LBuiltin(FPutPlanetDefense, 2, 2));
    LSymbol::define("put-planet-cargo", new LBuiltin(FPutPlanetCargo, 3, 3));
    LSymbol::define("put-planet-core", new LBuiltin(FPutPlanetCore, 3, 3));
    LSymbol::define("put-planet-density", new LBuiltin(FPutPlanetDensity, 3, 3));
    LSymbol::define("put-planet-colonist-tax", new LBuiltin(FPutPlanetColonistTax, 2, 2));
    LSymbol::define("put-planet-native-tax", new LBuiltin(FPutPlanetNativeTax, 2, 2));
    LSymbol::define("put-planet-colonist-happy", new LBuiltin(FPutPlanetColonistHappy, 2, 2));
    LSymbol::define("put-planet-native-happy", new LBuiltin(FPutPlanetNativeHappy, 2, 2));
    LSymbol::define("put-planet-native-gov", new LBuiltin(FPutPlanetNativeGov, 2, 2));
    LSymbol::define("put-planet-native-pop", new LBuiltin(FPutPlanetNativePop, 2, 2));
    LSymbol::define("put-planet-native-race", new LBuiltin(FPutPlanetNativeRace, 2, 2));
    LSymbol::define("put-planet-temp", new LBuiltin(FPutPlanetTemp, 2, 2));
    LSymbol::define("put-planet-build-base-p", new LBuiltin(FPutPlanetBuildBaseP, 2, 2));
    LSymbol::define("put-planet-name", new LBuiltin(FPutPlanetName, 2, 2));
    LSymbol::define("put-planet-location-x", new LBuiltin(FPutPlanetLocationX, 2, 2));
    LSymbol::define("put-planet-location-y", new LBuiltin(FPutPlanetLocationY, 2, 2));
    LSymbol::define("player-long-name", new LBuiltin(FPlayerLongName, 1, 1));
    LSymbol::define("player-short-name", new LBuiltin(FPlayerShortName, 1, 1));
    LSymbol::define("player-adj-name", new LBuiltin(FPlayerAdjName, 1, 1));
    LSymbol::define("put-player-long-name", new LBuiltin(FPutPlayerLongName, 2, 2));
    LSymbol::define("put-player-short-name", new LBuiltin(FPutPlayerShortName, 2, 2));
    LSymbol::define("put-player-adj-name", new LBuiltin(FPutPlayerAdjName, 2, 2));
    LSymbol::define("turn-time", new LBuiltin(FTurnTime, 0, 0));
    LSymbol::define("turn-number", new LBuiltin(FTurnNumber, 0, 0));
    LSymbol::define("make-fcode-special", new LBuiltin(FMakeFcodeSpecial, 1, 1));
    LSymbol::define("fcode-special-p", new LBuiltin(FFcodeSpecialP, 1, 1));
    LSymbol::define("ship-fcode-special-p", new LBuiltin(FShipFcodeSpecialP, 1, 1));
    LSymbol::define("planet-fcode-special-p", new LBuiltin(FPlanetFcodeSpecialP, 1, 1));
    LSymbol::define("fcode-match-p", new LBuiltin(FFcodeMatchP, 2, 2));
    LSymbol::define("tons-add", new LBuiltin(FTonsAdd, 3, 3));
    LSymbol::define("player-tons", new LBuiltin(FPlayerTons, 2, 2));
    LSymbol::define("put-player-tons", new LBuiltin(FPutPlayerTons, 3, 3));
    LSymbol::define("tons-clear-this-turn", new LBuiltin(FTonsClearThisTurn, 0, 0));
    LSymbol::define("tons-read", new LBuiltin(FTonsRead, 0, 0));
    LSymbol::define("tons-write", new LBuiltin(FTonsWrite, 0, 0));
    LSymbol::define("ship-has-enemy", new LBuiltin(FShipHasEnemy, 2, 2));
    LSymbol::define("player-has-enemy", new LBuiltin(FPlayerHasEnemy, 2, 2));
    LSymbol::define("put-player-has-enemy", new LBuiltin(FPutPlayerHasEnemy, 3, 3));
    LSymbol::define("player-enemy-supported-p", new LBuiltin(FPlayerEnemySupportedP, 0, 0));
    LSymbol::define("distance-squared", new LBuiltin(FDistanceSquared, 4, 4));
    LSymbol::define("distance", new LBuiltin(FDistance, 4, 4));
    LSymbol::define("distance-lt-radius-p", new LBuiltin(FDistanceLtRadiusP, 5, 5));
    LSymbol::define("wrap-x", new LBuiltin(FWrapX, 1, 1));
    LSymbol::define("wrap-y", new LBuiltin(FWrapY, 1, 1));
    LSymbol::define("wrap-dx", new LBuiltin(FWrapDx, 1, 1));
    LSymbol::define("wrap-dy", new LBuiltin(FWrapDy, 1, 1));
    LSymbol::define("point-on-map-p", new LBuiltin(FPointOnMapP, 2, 2));
    LSymbol::define("rewrap-ships-and-mines", new LBuiltin(FRewrapShipsAndMines, 0, 0));
    LSymbol::define("num-minefields", new LBuiltin(FNumMinefields, 0, 0));
    LSymbol::define("minefield-delete", new LBuiltin(FMinefieldDelete, 1, 1));
    LSymbol::define("minefield-exists-p", new LBuiltin(FMinefieldExistsP, 1, 1));
    LSymbol::define("minefield-radius", new LBuiltin(FMinefieldRadius, 1, 1));
    LSymbol::define("minefield-location-x", new LBuiltin(FMinefieldLocationX, 1, 1));
    LSymbol::define("minefield-location-y", new LBuiltin(FMinefieldLocationY, 1, 1));
    LSymbol::define("minefield-owner", new LBuiltin(FMinefieldOwner, 1, 1));
    LSymbol::define("minefield-units", new LBuiltin(FMinefieldUnits, 1, 1));
    LSymbol::define("minefield-web-p", new LBuiltin(FMinefieldWebP, 1, 1));
    LSymbol::define("put-minefield-units", new LBuiltin(FPutMinefieldUnits, 2, 2));
    LSymbol::define("put-minefield-owner", new LBuiltin(FPutMinefieldOwner, 2, 2));
    LSymbol::define("put-minefield-location-x", new LBuiltin(FPutMinefieldLocationX, 2, 2));
    LSymbol::define("put-minefield-location-y", new LBuiltin(FPutMinefieldLocationY, 2, 2));
    LSymbol::define("put-minefield-web-p", new LBuiltin(FPutMinefieldWebP, 2, 2));
    LSymbol::define("make-minefield", new LBuiltin(FMakeMinefield, 5, 5));
    LSymbol::define("base-delete", new LBuiltin(FBaseDelete, 1, 1));
    LSymbol::define("base-exists-p", new LBuiltin(FBaseExistsP, 1, 1));
    LSymbol::define("base-present-p", new LBuiltin(FBasePresentP, 1, 1));
    LSymbol::define("base-owner", new LBuiltin(FBaseOwner, 1, 1));
    LSymbol::define("base-defense", new LBuiltin(FBaseDefense, 1, 1));
    LSymbol::define("base-damage", new LBuiltin(FBaseDamage, 1, 1));
    LSymbol::define("base-mission", new LBuiltin(FBaseMission, 1, 1));
    LSymbol::define("base-fix-order", new LBuiltin(FBaseFixOrder, 1, 1));
    LSymbol::define("base-fix-ship", new LBuiltin(FBaseFixShip, 1, 1));
    LSymbol::define("base-fighters", new LBuiltin(FBaseFighters, 1, 1));
    LSymbol::define("base-tech", new LBuiltin(FBaseTech, 2, 2));
    LSymbol::define("base-hulls", new LBuiltin(FBaseHulls, 2, 2));
    LSymbol::define("base-engines", new LBuiltin(FBaseEngines, 2, 2));
    LSymbol::define("base-beams", new LBuiltin(FBaseBeams, 2, 2));
    LSymbol::define("base-tubes", new LBuiltin(FBaseTubes, 2, 2));
    LSymbol::define("base-torps", new LBuiltin(FBaseTorps, 2, 2));
    LSymbol::define("base-build-order", new LBuiltin(FBaseBuildOrder, 1, 1));
    LSymbol::define("make-base", new LBuiltin(FMakeBase, 1, 1));
    LSymbol::define("put-base-owner", new LBuiltin(FPutBaseOwner, 2, 2));
    LSymbol::define("put-base-defense", new LBuiltin(FPutBaseDefense, 2, 2));
    LSymbol::define("put-base-damage", new LBuiltin(FPutBaseDamage, 2, 2));
    LSymbol::define("put-base-fighters", new LBuiltin(FPutBaseFighters, 2, 2));
    LSymbol::define("put-base-mission", new LBuiltin(FPutBaseMission, 2, 2));
    LSymbol::define("put-base-fix-order", new LBuiltin(FPutBaseFixOrder, 2, 2));
    LSymbol::define("put-base-fix-ship", new LBuiltin(FPutBaseFixShip, 2, 2));
    LSymbol::define("put-base-tech", new LBuiltin(FPutBaseTech, 3, 3));
    LSymbol::define("put-base-hulls", new LBuiltin(FPutBaseHulls, 3, 3));
    LSymbol::define("put-base-engines", new LBuiltin(FPutBaseEngines, 3, 3));
    LSymbol::define("put-base-beams", new LBuiltin(FPutBaseBeams, 3, 3));
    LSymbol::define("put-base-tubes", new LBuiltin(FPutBaseTubes, 3, 3));
    LSymbol::define("put-base-torps", new LBuiltin(FPutBaseTorps, 3, 3));
    LSymbol::define("put-base-build-order", new LBuiltin(FPutBaseBuildOrder, 2, 2));
    LSymbol::define("base-clear-build-order", new LBuiltin(FBaseClearBuildOrder, 1, 1));
    LSymbol::define("base-clear-hulls", new LBuiltin(FBaseClearHulls, 1, 1));
    LSymbol::define("engine-name", new LBuiltin(FEngineName, 1, 1));
    LSymbol::define("engine-cost-mc", new LBuiltin(FEngineCostMc, 1, 1));
    LSymbol::define("engine-cost-t", new LBuiltin(FEngineCostT, 1, 1));
    LSymbol::define("engine-cost-d", new LBuiltin(FEngineCostD, 1, 1));
    LSymbol::define("engine-cost-m", new LBuiltin(FEngineCostM, 1, 1));
    LSymbol::define("engine-tech", new LBuiltin(FEngineTech, 1, 1));
    LSymbol::define("engine-fuel-usage", new LBuiltin(FEngineFuelUsage, 2, 2));
    LSymbol::define("put-engine-name", new LBuiltin(FPutEngineName, 2, 2));
    LSymbol::define("put-engine-cost-mc", new LBuiltin(FPutEngineCostMc, 2, 2));
    LSymbol::define("put-engine-cost-t", new LBuiltin(FPutEngineCostT, 2, 2));
    LSymbol::define("put-engine-cost-d", new LBuiltin(FPutEngineCostD, 2, 2));
    LSymbol::define("put-engine-cost-m", new LBuiltin(FPutEngineCostM, 2, 2));
    LSymbol::define("put-engine-tech", new LBuiltin(FPutEngineTech, 2, 2));
    LSymbol::define("put-engine-fuel-usage", new LBuiltin(FPutEngineFuelUsage, 3, 3));
    LSymbol::define("torp-name", new LBuiltin(FTorpName, 1, 1));
    LSymbol::define("torp-cost", new LBuiltin(FTorpCost, 1, 1));
    LSymbol::define("torp-tube-cost-mc", new LBuiltin(FTorpTubeCostMc, 1, 1));
    LSymbol::define("torp-tube-cost-t", new LBuiltin(FTorpTubeCostT, 1, 1));
    LSymbol::define("torp-tube-cost-d", new LBuiltin(FTorpTubeCostD, 1, 1));
    LSymbol::define("torp-tube-cost-m", new LBuiltin(FTorpTubeCostM, 1, 1));
    LSymbol::define("torp-tube-mass", new LBuiltin(FTorpTubeMass, 1, 1));
    LSymbol::define("torp-tech", new LBuiltin(FTorpTech, 1, 1));
    LSymbol::define("torp-kill", new LBuiltin(FTorpKill, 1, 1));
    LSymbol::define("torp-expl", new LBuiltin(FTorpExpl, 1, 1));
    LSymbol::define("put-torp-name", new LBuiltin(FPutTorpName, 2, 2));
    LSymbol::define("put-torp-cost", new LBuiltin(FPutTorpCost, 2, 2));
    LSymbol::define("put-torp-tube-cost-mc", new LBuiltin(FPutTorpTubeCostMc, 2, 2));
    LSymbol::define("put-torp-tube-cost-t", new LBuiltin(FPutTorpTubeCostT, 2, 2));
    LSymbol::define("put-torp-tube-cost-d", new LBuiltin(FPutTorpTubeCostD, 2, 2));
    LSymbol::define("put-torp-tube-cost-m", new LBuiltin(FPutTorpTubeCostM, 2, 2));
    LSymbol::define("put-torp-tube-mass", new LBuiltin(FPutTorpTubeMass, 2, 2));
    LSymbol::define("put-torp-tech", new LBuiltin(FPutTorpTech, 2, 2));
    LSymbol::define("put-torp-kill", new LBuiltin(FPutTorpKill, 2, 2));
    LSymbol::define("put-torp-expl", new LBuiltin(FPutTorpExpl, 2, 2));
    LSymbol::define("beam-name", new LBuiltin(FBeamName, 1, 1));
    LSymbol::define("beam-cost-mc", new LBuiltin(FBeamCostMc, 1, 1));
    LSymbol::define("beam-cost-t", new LBuiltin(FBeamCostT, 1, 1));
    LSymbol::define("beam-cost-d", new LBuiltin(FBeamCostD, 1, 1));
    LSymbol::define("beam-cost-m", new LBuiltin(FBeamCostM, 1, 1));
    LSymbol::define("beam-mass", new LBuiltin(FBeamMass, 1, 1));
    LSymbol::define("beam-tech", new LBuiltin(FBeamTech, 1, 1));
    LSymbol::define("beam-kill", new LBuiltin(FBeamKill, 1, 1));
    LSymbol::define("beam-expl", new LBuiltin(FBeamExpl, 1, 1));
    LSymbol::define("put-beam-name", new LBuiltin(FPutBeamName, 2, 2));
    LSymbol::define("put-beam-cost-mc", new LBuiltin(FPutBeamCostMc, 2, 2));
    LSymbol::define("put-beam-cost-t", new LBuiltin(FPutBeamCostT, 2, 2));
    LSymbol::define("put-beam-cost-d", new LBuiltin(FPutBeamCostD, 2, 2));
    LSymbol::define("put-beam-cost-m", new LBuiltin(FPutBeamCostM, 2, 2));
    LSymbol::define("put-beam-mass", new LBuiltin(FPutBeamMass, 2, 2));
    LSymbol::define("put-beam-tech", new LBuiltin(FPutBeamTech, 2, 2));
    LSymbol::define("put-beam-kill", new LBuiltin(FPutBeamKill, 2, 2));
    LSymbol::define("put-beam-expl", new LBuiltin(FPutBeamExpl, 2, 2));
    LSymbol::define("hull-name", new LBuiltin(FHullName, 1, 1));
    LSymbol::define("hull-picture", new LBuiltin(FHullPicture, 1, 1));
    LSymbol::define("hull-cost-t", new LBuiltin(FHullCostT, 1, 1));
    LSymbol::define("hull-cost-d", new LBuiltin(FHullCostD, 1, 1));
    LSymbol::define("hull-cost-m", new LBuiltin(FHullCostM, 1, 1));
    LSymbol::define("hull-max-fuel", new LBuiltin(FHullMaxFuel, 1, 1));
    LSymbol::define("hull-max-crew", new LBuiltin(FHullMaxCrew, 1, 1));
    LSymbol::define("hull-engine-number", new LBuiltin(FHullEngineNumber, 1, 1));
    LSymbol::define("hull-mass", new LBuiltin(FHullMass, 1, 1));
    LSymbol::define("hull-tech", new LBuiltin(FHullTech, 1, 1));
    LSymbol::define("hull-max-cargo", new LBuiltin(FHullMaxCargo, 1, 1));
    LSymbol::define("hull-bays", new LBuiltin(FHullBays, 1, 1));
    LSymbol::define("hull-max-tubes", new LBuiltin(FHullMaxTubes, 1, 1));
    LSymbol::define("hull-max-beams", new LBuiltin(FHullMaxBeams, 1, 1));
    LSymbol::define("hull-cost-mc", new LBuiltin(FHullCostMc, 1, 1));
    LSymbol::define("put-hull-name", new LBuiltin(FPutHullName, 2, 2));
    LSymbol::define("put-hull-picture", new LBuiltin(FPutHullPicture, 2, 2));
    LSymbol::define("put-hull-cost-t", new LBuiltin(FPutHullCostT, 2, 2));
    LSymbol::define("put-hull-cost-d", new LBuiltin(FPutHullCostD, 2, 2));
    LSymbol::define("put-hull-cost-m", new LBuiltin(FPutHullCostM, 2, 2));
    LSymbol::define("put-hull-max-fuel", new LBuiltin(FPutHullMaxFuel, 2, 2));
    LSymbol::define("put-hull-max-crew", new LBuiltin(FPutHullMaxCrew, 2, 2));
    LSymbol::define("put-hull-engine-number", new LBuiltin(FPutHullEngineNumber, 2, 2));
    LSymbol::define("put-hull-mass", new LBuiltin(FPutHullMass, 2, 2));
    LSymbol::define("put-hull-tech", new LBuiltin(FPutHullTech, 2, 2));
    LSymbol::define("put-hull-max-cargo", new LBuiltin(FPutHullMaxCargo, 2, 2));
    LSymbol::define("put-hull-bays", new LBuiltin(FPutHullBays, 2, 2));
    LSymbol::define("put-hull-max-tubes", new LBuiltin(FPutHullMaxTubes, 2, 2));
    LSymbol::define("put-hull-max-beams", new LBuiltin(FPutHullMaxBeams, 2, 2));
    LSymbol::define("put-hull-cost-mc", new LBuiltin(FPutHullCostMc, 2, 2));
    LSymbol::define("player-hull", new LBuiltin(FPlayerHull, 2, 2));
    LSymbol::define("put-player-hull", new LBuiltin(FPutPlayerHull, 3, 3));
    LSymbol::define("write-external-message", new LBuiltin(FWriteExternalMessage, 2, 2));
    LSymbol::define("write-auxhost-message", new LBuiltin(FWriteAuxhostMessage, 2, 2));
    LSymbol::define("random-init", new LBuiltin(FRandomInit, 1, 1));
    LSymbol::define("random-range", new LBuiltin(FRandomRange, 1, 1));
    LSymbol::define("ship-rc-start", new LBuiltin(FShipRcStart, 2, 2));
    LSymbol::define("ship-rc-end", new LBuiltin(FShipRcEnd, 2, 2));
    LSymbol::define("ship-rc-p", new LBuiltin(FShipRcP, 1, 1));
    LSymbol::define("ship-rc-allowed-p", new LBuiltin(FShipRcAllowedP, 1, 1));
    LSymbol::define("ship-rc-reset", new LBuiltin(FShipRcReset, 1, 1));
    LSymbol::define("ship-rc-set-default", new LBuiltin(FShipRcSetDefault, 1, 1));
    LSymbol::define("put-ship-rc-allowed-p", new LBuiltin(FPutShipRcAllowedP, 2, 2));
    LSymbol::define("ship-rc-controller", new LBuiltin(FShipRcController, 1, 1));
    LSymbol::define("ship-rc-owner", new LBuiltin(FShipRcOwner, 1, 1));
    LSymbol::define("player-rc-default", new LBuiltin(FPlayerRcDefault, 1, 1));
    LSymbol::define("put-player-rc-default", new LBuiltin(FPutPlayerRcDefault, 2, 2));
    LSymbol::define("worm-count", new LBuiltin(FWormCount, 0, 0));
    LSymbol::define("make-worm", new LBuiltin(FMakeWorm, 0, 0));
    LSymbol::define("worm-delete", new LBuiltin(FWormDelete, 1, 1));
    LSymbol::define("worm-stability", new LBuiltin(FWormStability, 1, 1));
    LSymbol::define("worm-start-x", new LBuiltin(FWormStartX, 1, 1));
    LSymbol::define("worm-start-y", new LBuiltin(FWormStartY, 1, 1));
    LSymbol::define("worm-end-x", new LBuiltin(FWormEndX, 1, 1));
    LSymbol::define("worm-end-y", new LBuiltin(FWormEndY, 1, 1));
    LSymbol::define("worm-mass", new LBuiltin(FWormMass, 1, 1));
    LSymbol::define("worm-instability", new LBuiltin(FWormInstability, 1, 1));
    LSymbol::define("worm-start-waypoint-x", new LBuiltin(FWormStartWaypointX, 1, 1));
    LSymbol::define("worm-start-waypoint-y", new LBuiltin(FWormStartWaypointY, 1, 1));
    LSymbol::define("worm-end-waypoint-x", new LBuiltin(FWormEndWaypointX, 1, 1));
    LSymbol::define("worm-end-waypoint-y", new LBuiltin(FWormEndWaypointY, 1, 1));
    LSymbol::define("put-worm-start-x", new LBuiltin(FPutWormStartX, 2, 2));
    LSymbol::define("put-worm-start-y", new LBuiltin(FPutWormStartY, 2, 2));
    LSymbol::define("put-worm-end-x", new LBuiltin(FPutWormEndX, 2, 2));
    LSymbol::define("put-worm-end-y", new LBuiltin(FPutWormEndY, 2, 2));
    LSymbol::define("put-worm-mass", new LBuiltin(FPutWormMass, 2, 2));
    LSymbol::define("put-worm-instability", new LBuiltin(FPutWormInstability, 2, 2));
    LSymbol::define("put-worm-start-waypoint-x", new LBuiltin(FPutWormStartWaypointX, 2, 2));
    LSymbol::define("put-worm-start-waypoint-y", new LBuiltin(FPutWormStartWaypointY, 2, 2));
    LSymbol::define("put-worm-end-waypoint-x", new LBuiltin(FPutWormEndWaypointX, 2, 2));
    LSymbol::define("put-worm-end-waypoint-y", new LBuiltin(FPutWormEndWaypointY, 2, 2));
    LSymbol::define("ship-exp-level", new LBuiltin(FShipExpLevel, 1, 1));
    LSymbol::define("ship-exp-add", new LBuiltin(FShipExpAdd, 2, 2));
    LSymbol::define("ship-exp-points", new LBuiltin(FShipExpPoints, 1, 1));
    LSymbol::define("put-ship-exp-points", new LBuiltin(FPutShipExpPoints, 2, 2));
    LSymbol::define("ship-exp-new-points", new LBuiltin(FShipExpNewPoints, 1, 1));
    LSymbol::define("planet-exp-level", new LBuiltin(FPlanetExpLevel, 1, 1));
    LSymbol::define("planet-exp-add", new LBuiltin(FPlanetExpAdd, 2, 2));
    LSymbol::define("planet-exp-points", new LBuiltin(FPlanetExpPoints, 1, 1));
    LSymbol::define("put-planet-exp-points", new LBuiltin(FPutPlanetExpPoints, 2, 2));
    LSymbol::define("planet-exp-new-points", new LBuiltin(FPlanetExpNewPoints, 1, 1));
    LSymbol::define("load-game", new LBuiltin(FLoadGame, 0, 0));
    LSymbol::define("load-spec", new LBuiltin(FLoadSpec, 0, 0));
    LSymbol::define("save-game", new LBuiltin(FSaveGame, 0, 0));
    LSymbol::define("save-planets", new LBuiltin(FSavePlanets, 0, 0));
    LSymbol::define("save-ships", new LBuiltin(FSaveShips, 0, 0));
    LSymbol::define("save-bases", new LBuiltin(FSaveBases, 0, 0));
    LSymbol::define("save-mines", new LBuiltin(FSaveMines, 0, 0));
    LSymbol::define("save-gen", new LBuiltin(FSaveGen, 0, 0));
    LSymbol::define("save-aux", new LBuiltin(FSaveAux, 0, 0));
    LSymbol::define("save-racenames", new LBuiltin(FSaveRacenames, 0, 0));
    LSymbol::define("save-wormholes", new LBuiltin(FSaveWormholes, 0, 0));
    LSymbol::define("save-cloak-status", new LBuiltin(FSaveCloakStatus, 0, 0));
    LSymbol::define("save-hullspec", new LBuiltin(FSaveHullspec, 0, 0));
    LSymbol::define("save-truehull", new LBuiltin(FSaveTruehull, 0, 0));
    LSymbol::define("save-engspec", new LBuiltin(FSaveEngspec, 0, 0));
    LSymbol::define("save-torpspec", new LBuiltin(FSaveTorpspec, 0, 0));
    LSymbol::define("save-beamspec", new LBuiltin(FSaveBeamspec, 0, 0));
    LSymbol::define("save-planet-names", new LBuiltin(FSavePlanetNames, 0, 0));
    LSymbol::define("save-planet-locations", new LBuiltin(FSavePlanetLocations, 0, 0));
    LSymbol::define("ship-cloak-status-known-p", new LBuiltin(FShipCloakStatusKnownP, 0, 0));
    LSymbol::define("init-planets", new LBuiltin(FInitPlanets, 0, 0));
    LSymbol::define("init-ships", new LBuiltin(FInitShips, 0, 0));
    LSymbol::define("init-bases", new LBuiltin(FInitBases, 0, 0));
    LSymbol::define("init-mines", new LBuiltin(FInitMines, 0, 0));
    LSymbol::define("init-gen", new LBuiltin(FInitGen, 0, 0));
    LSymbol::define("init-version", new LBuiltin(FInitVersion, 1, 1));
    LSymbol::define("util-set-mode", new LBuiltin(FUtilSetMode, 1, 1));
    LSymbol::define("game-file-open-read", new LBuiltin(FGameFileOpenRead, 2, 2));
    LSymbol::define("game-file-open-write", new LBuiltin(FGameFileOpenWrite, 2, 2));
    LSymbol::define("game-file-remove", new LBuiltin(FGameFileRemove, 1, 1));
    LSymbol::define("game-file-copy", new LBuiltin(FGameFileCopy, 2, 2));
    LSymbol::define("game-file-rename", new LBuiltin(FGameFileRename, 2, 2));
    LSymbol::define("file-size", new LBuiltin(FFileSize, 1, 1));
    LSymbol::defineSetf("activity-level", "put-activity-level");
    LSymbol::defineSetf("base-beams", "put-base-beams");
    LSymbol::defineSetf("base-build-order", "put-base-build-order");
    LSymbol::defineSetf("base-damage", "put-base-damage");
    LSymbol::defineSetf("base-defense", "put-base-defense");
    LSymbol::defineSetf("base-engines", "put-base-engines");
    LSymbol::defineSetf("base-fighters", "put-base-fighters");
    LSymbol::defineSetf("base-fix-order", "put-base-fix-order");
    LSymbol::defineSetf("base-fix-ship", "put-base-fix-ship");
    LSymbol::defineSetf("base-hulls", "put-base-hulls");
    LSymbol::defineSetf("base-mission", "put-base-mission");
    LSymbol::defineSetf("base-owner", "put-base-owner");
    LSymbol::defineSetf("base-tech", "put-base-tech");
    LSymbol::defineSetf("base-torps", "put-base-torps");
    LSymbol::defineSetf("base-tubes", "put-base-tubes");
    LSymbol::defineSetf("beam-cost-d", "put-beam-cost-d");
    LSymbol::defineSetf("beam-cost-m", "put-beam-cost-m");
    LSymbol::defineSetf("beam-cost-mc", "put-beam-cost-mc");
    LSymbol::defineSetf("beam-cost-t", "put-beam-cost-t");
    LSymbol::defineSetf("beam-expl", "put-beam-expl");
    LSymbol::defineSetf("beam-kill", "put-beam-kill");
    LSymbol::defineSetf("beam-mass", "put-beam-mass");
    LSymbol::defineSetf("beam-name", "put-beam-name");
    LSymbol::defineSetf("beam-tech", "put-beam-tech");
    LSymbol::defineSetf("build-queue-priority", "put-build-queue-priority");
    LSymbol::defineSetf("engine-cost-d", "put-engine-cost-d");
    LSymbol::defineSetf("engine-cost-m", "put-engine-cost-m");
    LSymbol::defineSetf("engine-cost-mc", "put-engine-cost-mc");
    LSymbol::defineSetf("engine-cost-t", "put-engine-cost-t");
    LSymbol::defineSetf("engine-fuel-usage", "put-engine-fuel-usage");
    LSymbol::defineSetf("engine-name", "put-engine-name");
    LSymbol::defineSetf("engine-tech", "put-engine-tech");
    LSymbol::defineSetf("hull-bays", "put-hull-bays");
    LSymbol::defineSetf("hull-cost-d", "put-hull-cost-d");
    LSymbol::defineSetf("hull-cost-m", "put-hull-cost-m");
    LSymbol::defineSetf("hull-cost-mc", "put-hull-cost-mc");
    LSymbol::defineSetf("hull-cost-t", "put-hull-cost-t");
    LSymbol::defineSetf("hull-engine-number", "put-hull-engine-number");
    LSymbol::defineSetf("hull-mass", "put-hull-mass");
    LSymbol::defineSetf("hull-max-beams", "put-hull-max-beams");
    LSymbol::defineSetf("hull-max-cargo", "put-hull-max-cargo");
    LSymbol::defineSetf("hull-max-crew", "put-hull-max-crew");
    LSymbol::defineSetf("hull-max-fuel", "put-hull-max-fuel");
    LSymbol::defineSetf("hull-max-tubes", "put-hull-max-tubes");
    LSymbol::defineSetf("hull-name", "put-hull-name");
    LSymbol::defineSetf("hull-picture", "put-hull-picture");
    LSymbol::defineSetf("hull-tech", "put-hull-tech");
    LSymbol::defineSetf("kill-config", "put-kill-config");
    LSymbol::defineSetf("minefield-location-x", "put-minefield-location-x");
    LSymbol::defineSetf("minefield-location-y", "put-minefield-location-y");
    LSymbol::defineSetf("minefield-owner", "put-minefield-owner");
    LSymbol::defineSetf("minefield-units", "put-minefield-units");
    LSymbol::defineSetf("minefield-web-p", "put-minefield-web-p");
    LSymbol::defineSetf("planet-build-base-p", "put-planet-build-base-p");
    LSymbol::defineSetf("planet-cargo", "put-planet-cargo");
    LSymbol::defineSetf("planet-colonist-happy", "put-planet-colonist-happy");
    LSymbol::defineSetf("planet-colonist-tax", "put-planet-colonist-tax");
    LSymbol::defineSetf("planet-core", "put-planet-core");
    LSymbol::defineSetf("planet-defense", "put-planet-defense");
    LSymbol::defineSetf("planet-density", "put-planet-density");
    LSymbol::defineSetf("planet-exp-points", "put-planet-exp-points");
    LSymbol::defineSetf("planet-factories", "put-planet-factories");
    LSymbol::defineSetf("planet-fcode", "put-planet-fcode");
    LSymbol::defineSetf("planet-location-x", "put-planet-location-x");
    LSymbol::defineSetf("planet-location-y", "put-planet-location-y");
    LSymbol::defineSetf("planet-mines", "put-planet-mines");
    LSymbol::defineSetf("planet-name", "put-planet-name");
    LSymbol::defineSetf("planet-native-gov", "put-planet-native-gov");
    LSymbol::defineSetf("planet-native-happy", "put-planet-native-happy");
    LSymbol::defineSetf("planet-native-pop", "put-planet-native-pop");
    LSymbol::defineSetf("planet-native-race", "put-planet-native-race");
    LSymbol::defineSetf("planet-native-tax", "put-planet-native-tax");
    LSymbol::defineSetf("planet-owner", "put-planet-owner");
    LSymbol::defineSetf("planet-temp", "put-planet-temp");
    LSymbol::defineSetf("player-active-p", "put-player-active-p");
    LSymbol::defineSetf("player-adj-name", "put-player-adj-name");
    LSymbol::defineSetf("player-has-enemy", "put-player-has-enemy");
    LSymbol::defineSetf("player-hull", "put-player-hull");
    LSymbol::defineSetf("player-long-name", "put-player-long-name");
    LSymbol::defineSetf("player-password", "put-player-password");
    LSymbol::defineSetf("player-rc-default", "put-player-rc-default");
    LSymbol::defineSetf("player-short-name", "put-player-short-name");
    LSymbol::defineSetf("player-tons", "put-player-tons");
    LSymbol::defineSetf("ship-ammo", "put-ship-ammo");
    LSymbol::defineSetf("ship-bays", "put-ship-bays");
    LSymbol::defineSetf("ship-beam-number", "put-ship-beam-number");
    LSymbol::defineSetf("ship-beam-type", "put-ship-beam-type");
    LSymbol::defineSetf("ship-cargo", "put-ship-cargo");
    LSymbol::defineSetf("ship-cloaked-p", "put-ship-cloaked-p");
    LSymbol::defineSetf("ship-crew", "put-ship-crew");
    LSymbol::defineSetf("ship-damage", "put-ship-damage");
    LSymbol::defineSetf("ship-enemy", "put-ship-enemy");
    LSymbol::defineSetf("ship-engine", "put-ship-engine");
    LSymbol::defineSetf("ship-exp-points", "put-ship-exp-points");
    LSymbol::defineSetf("ship-fcode", "put-ship-fcode");
    LSymbol::defineSetf("ship-hull", "put-ship-hull");
    LSymbol::defineSetf("ship-intercept-target", "put-ship-intercept-target");
    LSymbol::defineSetf("ship-location-x", "put-ship-location-x");
    LSymbol::defineSetf("ship-location-y", "put-ship-location-y");
    LSymbol::defineSetf("ship-mission", "put-ship-mission");
    LSymbol::defineSetf("ship-name", "put-ship-name");
    LSymbol::defineSetf("ship-owner", "put-ship-owner");
    LSymbol::defineSetf("ship-rc-allowed-p", "put-ship-rc-allowed-p");
    LSymbol::defineSetf("ship-shield-level", "put-ship-shield-level");
    LSymbol::defineSetf("ship-speed", "put-ship-speed");
    LSymbol::defineSetf("ship-torp-type", "put-ship-torp-type");
    LSymbol::defineSetf("ship-tow-target", "put-ship-tow-target");
    LSymbol::defineSetf("ship-transfer-cargo", "put-ship-transfer-cargo");
    LSymbol::defineSetf("ship-transfer-target", "put-ship-transfer-target");
    LSymbol::defineSetf("ship-tube-number", "put-ship-tube-number");
    LSymbol::defineSetf("ship-unload-cargo", "put-ship-unload-cargo");
    LSymbol::defineSetf("ship-unload-target", "put-ship-unload-target");
    LSymbol::defineSetf("ship-waypoint-dx", "put-ship-waypoint-dx");
    LSymbol::defineSetf("ship-waypoint-dy", "put-ship-waypoint-dy");
    LSymbol::defineSetf("ship-waypoint-x", "put-ship-waypoint-x");
    LSymbol::defineSetf("ship-waypoint-y", "put-ship-waypoint-y");
    LSymbol::defineSetf("torp-cost", "put-torp-cost");
    LSymbol::defineSetf("torp-expl", "put-torp-expl");
    LSymbol::defineSetf("torp-kill", "put-torp-kill");
    LSymbol::defineSetf("torp-name", "put-torp-name");
    LSymbol::defineSetf("torp-tech", "put-torp-tech");
    LSymbol::defineSetf("torp-tube-cost-d", "put-torp-tube-cost-d");
    LSymbol::defineSetf("torp-tube-cost-m", "put-torp-tube-cost-m");
    LSymbol::defineSetf("torp-tube-cost-mc", "put-torp-tube-cost-mc");
    LSymbol::defineSetf("torp-tube-cost-t", "put-torp-tube-cost-t");
    LSymbol::defineSetf("torp-tube-mass", "put-torp-tube-mass");
    LSymbol::defineSetf("worm-end-waypoint-x", "put-worm-end-waypoint-x");
    LSymbol::defineSetf("worm-end-waypoint-y", "put-worm-end-waypoint-y");
    LSymbol::defineSetf("worm-end-x", "put-worm-end-x");
    LSymbol::defineSetf("worm-end-y", "put-worm-end-y");
    LSymbol::defineSetf("worm-instability", "put-worm-instability");
    LSymbol::defineSetf("worm-mass", "put-worm-mass");
    LSymbol::defineSetf("worm-start-waypoint-x", "put-worm-start-waypoint-x");
    LSymbol::defineSetf("worm-start-waypoint-y", "put-worm-start-waypoint-y");
    LSymbol::defineSetf("worm-start-x", "put-worm-start-x");
    LSymbol::defineSetf("worm-start-y", "put-worm-start-y");
}

